//	2003->2007
//	Rename form
//	Word replace "2003" for "2007"
//	Rename namespace

using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using Infragistics.Shared;
using Infragistics.Win;
using Infragistics.Win.UltraWinSchedule;
using System.Diagnostics;
using System.Resources;
using System.Globalization;
using Infragistics.Win.Misc;
using Infragistics.Win.UltraWinEditors;

namespace AppointmentDialog2007_CS
{
	#region RecurrenceDialog2007 class
	/// <summary>
	/// Dialog used to modify the properties of an AppointmentRecurrence object.
	/// </summary>
	public class RecurrenceDialog2007 : System.Windows.Forms.Form
	{
		#region Constants
		static readonly int DIALOG_WIDTH = 468;
		private static readonly int DIALOG_HEIGHT = 351;

		private static readonly int PANEL_WIDTH = 350;
		private static readonly int PANEL_HEIGHT = 96;

		private static readonly int PANEL_LEFT = 80;
		private static readonly int PANEL_TOP = 16;

		static internal readonly TimeSpan									defaultOccurrenceDuration = TimeSpan.Zero;
		static internal readonly DateTime									defaultOccurrenceStartTime = DateTime.MinValue;
		static internal readonly int										defaultPatternDayOfMonth = 0;
		static internal readonly RecurrencePatternDaysOfWeek				defaultPatternDaysOfWeek = RecurrencePatternDaysOfWeek.None;
		static internal readonly RecurrencePatternFrequency					defaultPatternFrequency = RecurrencePatternFrequency.Weekly;
		static internal readonly int										defaultPatternInterval = 1;
		static internal readonly int										defaultPatternMonthOfYear = 0;
		static internal readonly RecurrencePatternOccurrenceOfDayInMonth	defaultPatternOccurrenceOfDayInMonth = RecurrencePatternOccurrenceOfDayInMonth.None;
		static internal readonly DateTime									defaultRangeEndDate = DateTime.MinValue;
		static internal readonly DateTime									defaultRangeStartDate = DateTime.MinValue;
		static internal readonly RecurrenceRangeLimit						defaultRangeLimit = RecurrenceRangeLimit.NoLimit;
		static internal readonly int										defaultRangeMaxOccurrences = 10;
		static internal readonly RecurrencePatternType						defaultPatternType = RecurrencePatternType.Explicit;

		static internal readonly int										maxPatternDayOfMonth = 31;
		static internal readonly int										maxPatternInterval = 99;
		static internal readonly int										maxPatternMonthOfYear = 12;
		static internal readonly int										maxMaxOccurrences = 999;

		static private readonly string RecurrenceDialog_Error_DurationCannotExceedFrequency = "The duration of the appointment must be shorter than how frequently it occurs. Shorten the duration, or change the recurrence pattern in the Appointment Recurrence dialog box.";
		static private readonly string RecurrenceDialog_Warning_PatternDayOfMonthExceeds28 = "Some months have fewer than the number of days specified. For these months, the occurrence will fall on the last day of the month.";
		static private readonly string RecurrenceDialog_Error_RecurrencePatterIsInvalid = "The recurrence pattern is not valid.";
		static private readonly string RecurrenceDialog_MessageBox_Caption = "Appointment Recurrence";
		static private readonly string RecurrenceDialog_Prompt_OkToRemoveVariances = "Any exceptions associated with this recurring appointment will be lost. Is this OK?";
		#endregion Constants



		#region Member variables
		
		private System.ComponentModel.Container components = null;
		private Infragistics.Win.Misc.UltraGroupBox fraAppointmentTime;
		private Infragistics.Win.Misc.UltraGroupBox fraRecurrencePattern;
		private Infragistics.Win.Misc.UltraGroupBox fraRecurrenceRange;
		private UltraButton cmdOk;
		private UltraButton cmdCancel;
		private UltraButton cmdRemoveRecurrence;
		private UltraLabel lblOccurrenceStartTime;
		private UltraComboEditor cboOccurrenceStartTime;
		private UltraLabel lblEndTime;
		private UltraLabel lblOccurrenceDuration;
		private UltraLabel lblSepLight;
		private UltraLabel lblSepDark;
		private System.Windows.Forms.Panel pnlWeekly;
		private System.Windows.Forms.Panel pnlMonthly;
		private System.Windows.Forms.Panel pnlDaily;
		private System.Windows.Forms.Panel pnlYearly;
		private UltraTextEditor txtPatternIntervalDaily;
		private UltraLabel lblPatternInterval1_Weekly;
		private UltraTextEditor txtPatternIntervalWeekly;
		private UltraCheckEditor chkSunday;
		private UltraCheckEditor chkMonday;
		private UltraCheckEditor chkTuesday;
		private UltraCheckEditor chkWednesday;
		private UltraCheckEditor chkThursday;
		private UltraCheckEditor chkFriday;
		private UltraCheckEditor chkSaturday;
		private UltraComboEditor cboOccurrenceOfDayInMonthMonthly;
		private UltraComboEditor cboDayOfWeekMonthly;
		private UltraTextEditor txtPatternIntervalExplicitMonthly;
		private UltraTextEditor txtPatternIntervalCalculatedMonthly;
		private UltraLabel lblCalculatedYearly;
		private UltraComboEditor cboDayOfWeekYearly;
		private UltraComboEditor cboOccurrenceOfDayInMonthYearly;
		private UltraLabel lblPatternInterval2Weekly;
		private UltraLabel lblPatternInterval4Monthly;
		private UltraLabel lblPatternInterval3Monthly;
		private UltraLabel lblPatternInterval2Monthly;
		private UltraLabel lblPatternInterval1Monthly;
		private UltraLabel lblPatternIntervalDaily;
		private UltraTextEditor txtRangeMaxOccurrences;
		private UltraLabel lblRangeMaxOccurrences;
		private UltraCalendarCombo dtpRangeEndDate;
		private UltraCalendarCombo dtpRangeStartDate;
		private UltraLabel lblRangeStartDate;
		private UltraComboEditor cboEndTime;

		private Appointment appointment = null;
		private AppointmentRecurrence appointmentRecurrence = null;
		private AppointmentRecurrence initialAppointmentRecurrence = null;

		private UltraComboEditor cboOccurrenceDuration;
		private UltraTextEditor txtPatternDayOfMonthMonthly;
		private UltraTextEditor txtPatternDayOfMonthYearly;
		private UltraComboEditor cboMonthOfYearCalculatedYearly;
		private UltraComboEditor cboMonthOfYearExplicitYearly;
		private bool suspendEventHandling = false;
		private bool suspendOptionButtonHandling = false;
		private bool suspendUpdate = false;
		private bool isInitializing = false;
		private RecurrenceDialogResult result = RecurrenceDialogResult.Cancel;
		private bool hasVariances = false;
		private bool forceContentsChanged = false;

		private bool contentsAreValid = true;
		private Infragistics.Win.UltraWinEditors.UltraOptionSet optPatternFrequency;
		private Infragistics.Win.UltraWinEditors.UltraOptionSet optRangeLimit;
		private Infragistics.Win.UltraWinEditors.UltraOptionSet optPatternDaysOfWeek_Daily;
		private Infragistics.Win.UltraWinEditors.UltraOptionSet optPatternType_Monthly;
		private Infragistics.Win.UltraWinEditors.UltraOptionSet optPatternType_Yearly;

		private bool patternDayOfMonthWarningDisplayed = false;
		private bool allowNoEndDate = true;

		#endregion Member variables

		#region Delegates
		private delegate void afterSelection_cboEndTime();
		#endregion Delegates

		#region Constructors
		/// <summary>
		/// Creates a new instance of the <see cref="RecurrenceDialog"/> class.
		/// </summary>
		/// <param name="appointment">The appointment on which the specified recurrence is to be based.</param>
		/// <param name="recurrence">The AppointmentRecurrence instance the dialog is editing.</param>
		/// <param name="allowNoEndDate">Determines whether the 'No End Date' option is displayed.</param>
		/// <param name="allowRemoveRecurrence">Determines whether the 'Remove Recurrence' Button is enabled.</param>
		/// <param name="hasVariances">Determines whether the specified recurrence should be treated as if it has variances</param>
		public RecurrenceDialog2007( Appointment appointment, AppointmentRecurrence recurrence, bool allowNoEndDate, bool allowRemoveRecurrence, bool hasVariances )
		{
			//	Throw an exception if either the appointment or the recurrence are null
			if ( appointment == null || recurrence == null )
				throw new ArgumentNullException();

			//	Flag the dialog as being in a state of initializing
			this.isInitializing = true;

			//	Set the 'appointment' member variable
			this.appointment = appointment;

			//	If the specified appointment is currently not a member
			//	of a recurrence, we want to consider the dialog dirty
			//	even if no dialog fields were modified, so that clicking
			//	Ok results in the recurrence being assigned to the appointment.
			if ( ! this.appointment.IsRecurringAppointmentRoot &&
				 this.appointment.RecurringAppointmentRoot == null )
				this.forceContentsChanged = true;

			//	Set the 'appointmentRecurrence' member variable
			AppointmentRecurrence recurrenceCopy = new AppointmentRecurrence();
			recurrenceCopy.InitializeFrom( recurrence, true );
			this.appointmentRecurrence = recurrenceCopy;

			//	Set the 'hasVariances' member variable
			this.hasVariances = hasVariances;

			//	Cache the recurrence, so we have something to compare
			//	the working one to in order to know if anything changed.
			this.initialAppointmentRecurrence = new AppointmentRecurrence();
			this.initialAppointmentRecurrence.InitializeFrom( this.appointmentRecurrence, false );

			this.InitializeComponent();

			//	If the 'allowRemoveRecurrence'parameter is false, disable the 'cmdRemoveRecurrence' button.
			this.cmdRemoveRecurrence.Enabled =  allowRemoveRecurrence;
			
			//	Cache the value of the 'allowNoEndDate' parameter
			this.allowNoEndDate = allowNoEndDate;

			if ( this.allowNoEndDate == false &&
				 this.appointmentRecurrence.RangeLimit == RecurrenceRangeLimit.NoLimit )
				this.appointmentRecurrence.RangeLimit = RecurrenceRangeLimit.LimitByNumberOfOccurrences;

			//	Hook the events of interest.
			this.HookDialogControlEvents( false );
			
			//	Initialize the dialog controls to reflect the property values
			//	of the AppointmentRecurrence that it is going to be editing
			this.InitializeDialogControls();

			//	Set some Form-specific properties
			this.MaximizeBox = true;
			this.MinimizeBox = true;
			this.ControlBox = false;
			this.FormBorderStyle = FormBorderStyle.FixedDialog;
			this.MaximizeBox = false;
			this.MinimizeBox = false;
			this.ControlBox = true;
			this.StartPosition = FormStartPosition.CenterParent;

			//	Set the Accept and Cancel buttons
			this.AcceptButton = this.cmdOk;
			this.CancelButton = this.cmdCancel;

			//	Set the size of the dialog
			this.Size = new Size( RecurrenceDialog2007.DIALOG_WIDTH, RecurrenceDialog2007.DIALOG_HEIGHT );

			//	Clear the 'isInitializing' flag.
			this.isInitializing = false;
		}
		#endregion Constructors

		#region Public Properties

			#region Recurrence
		/// <summary>
		/// (Read-only) Returns an <see cref="AppointmentRecurrence"/> instance
		/// that reflects the values entered by the end user during this dialog session.
		/// </summary>
		/// <remarks><p class="note"><b>Note:</b> This property is only valid when the dialog has been closed and the dialog session was not canceled.</p></remarks>
		public AppointmentRecurrence Recurrence
		{
			get
			{
				if ( this.Result == RecurrenceDialogResult.Cancel ||
					 this.Result == RecurrenceDialogResult.RemoveRecurrence )
					return null;

				this.UpdateRecurrence();
				return this.appointmentRecurrence;
			}
		}
			#endregion Recurrence

			#region Result
		/// <summary>
		/// Returns the result of the dialog's last user session.
		/// </summary>
		public RecurrenceDialogResult Result
		{
			get{ return this.result; }
		}
			#endregion Result

		#endregion Public Properties

		#region Internal Properties

			#region PanelSize
#if DEBUG
		/// <summary>
		/// Returns the size to be used for the Panel controls.
		/// </summary>
#endif
		internal Size PanelSize
		{
			get{ return new Size( RecurrenceDialog2007.PANEL_WIDTH, RecurrenceDialog2007.PANEL_HEIGHT ); }
		}
			#endregion PanelSize

			#region PanelLocation
#if DEBUG
		/// <summary>
		/// Returns the location to be used for the Panel controls.
		/// Location is relative to the fraRecurrencePattern GroupBox.
		/// </summary>
#endif
		internal Point PanelLocation
		{
			get{ return new Point( RecurrenceDialog2007.PANEL_LEFT, RecurrenceDialog2007.PANEL_TOP ); }
		}
			#endregion PanelSize
			
			#region Recurrence properties
		//
		//	The following region contains properties that have the same name
		//	as their AppointmentRecurrence counterparts. The get methods return
		//	a value that is based on the state of the dialog control that
		//	represents that property; the set methods set the state of the
		//	dialog control that corresponds to the property being set. This is
		//	an abstraction that is designed to make it easier to map AppointmentRecurrence
		//	properties to the UI, and get the values back again.
		//	
		//	Note that the set methods are not likely to throw an exception, but the gets
		//	quite possibly will (for example, if the contents of a TextBox is not a number,
		//	but we try to convert it as such). Recommended usage is to wrap the call to the
		//	property get in a try/catch, and display a meaningful error to the end user (usually
		//	"The recurrence pattern is not valid", as with MS Outlook.)

			#region RangeStartDate
#if DEBUG
		/// <summary>
		/// Gets/sets the RangeStartDate (dtpRangeStartDate)
		/// </summary>
#endif
		internal DateTime RangeStartDate
		{
			get{ return (DateTime)this.dtpRangeStartDate.Value; }
			set
			{ 
				this.dtpRangeStartDate.Value = value; 
			}
		}
			#endregion RangeStartDate

			#region RangeEndDate
#if DEBUG
		/// <summary>
		/// Gets/sets the RangeEndDate (dtpRangeEndDate)
		/// </summary>
#endif
		internal DateTime RangeEndDate
		{
			get{ return (DateTime)this.dtpRangeEndDate.Value; }
			set
			{ 
				this.dtpRangeEndDate.Value = value;
			}
		}
		#endregion RangeEndDate

			#region PatternFrequency
#if DEBUG
		/// <summary>
		/// Gets/sets the PatternFrequency (optDaily, optWeekly, optMonthly, optYearly)
		/// </summary>
#endif
		internal RecurrencePatternFrequency PatternFrequency
		{
			get
			{
				return (RecurrencePatternFrequency)this.optPatternFrequency.Value;
			}

			set
			{
				this.optPatternFrequency.Value = value;
			}
		}
			#endregion PatternFrequency

			#region RangeLimit
#if DEBUG
		/// <summary>
		/// Gets/sets the RangeLimit (optNoLimit, optLimitByNumberOfOccurrences, optNoLimit)
		/// </summary>
#endif
		internal RecurrenceRangeLimit RangeLimit
		{
			get
			{
				return (RecurrenceRangeLimit)this.optRangeLimit.Value;
			}

			set
			{
				this.optRangeLimit.Value = value;
			}
		}
			#endregion RangeLimit

			#region PatternDaysOfWeek
#if DEBUG
		/// <summary>
		/// Gets/sets the PatternDaysOfWeek
		/// (For Weekly: chkSunday - chkSaturday; for Daily: optAllDays, optAllWeekdays)
		/// </summary>
#endif
		internal RecurrencePatternDaysOfWeek PatternDaysOfWeek
		{
			get
			{
				if ( this.PatternFrequency == RecurrencePatternFrequency.Daily )
				{
					return (RecurrencePatternDaysOfWeek)this.optPatternDaysOfWeek_Daily.Value;
				}
				else
				if ( this.PatternFrequency == RecurrencePatternFrequency.Weekly )
				{
					//	Build a bitmask based on which checkboxes are checked.
					RecurrencePatternDaysOfWeek retVal = RecurrencePatternDaysOfWeek.None;
					if ( this.chkSunday.Checked )
						retVal |= RecurrencePatternDaysOfWeek.Sunday;
					if ( this.chkMonday.Checked )
						retVal |= RecurrencePatternDaysOfWeek.Monday;
					if ( this.chkTuesday.Checked )
						retVal |= RecurrencePatternDaysOfWeek.Tuesday;
					if ( this.chkWednesday.Checked )
						retVal |= RecurrencePatternDaysOfWeek.Wednesday;
					if ( this.chkThursday.Checked )
						retVal |= RecurrencePatternDaysOfWeek.Thursday;
					if ( this.chkFriday.Checked )
						retVal |= RecurrencePatternDaysOfWeek.Friday;
					if ( this.chkSaturday.Checked )
						retVal |= RecurrencePatternDaysOfWeek.Saturday;

					return retVal;
				}
				else
				if ( this.PatternFrequency == RecurrencePatternFrequency.Monthly )
				{
					ValueListItem item = this.cboDayOfWeekMonthly.SelectedItem as ValueListItem;
					return item != null ? (RecurrencePatternDaysOfWeek)item.DataValue : RecurrencePatternDaysOfWeek.None;
				}
				else
				if ( this.PatternFrequency == RecurrencePatternFrequency.Yearly )
				{
					ValueListItem item = this.cboDayOfWeekYearly.SelectedItem as ValueListItem;
					return item != null ? (RecurrencePatternDaysOfWeek)item.DataValue : RecurrencePatternDaysOfWeek.None;
				}

				return RecurrencePatternDaysOfWeek.None;
			}

			set
			{
				if ( this.PatternFrequency == RecurrencePatternFrequency.Daily )
				{
					if ( value == RecurrencePatternDaysOfWeek.AllWeekdays )
						this.optPatternDaysOfWeek_Daily.Value = RecurrencePatternDaysOfWeek.AllWeekdays;
					else
						this.optPatternDaysOfWeek_Daily.Value = RecurrencePatternDaysOfWeek.All;
				}
				else
				if ( this.PatternFrequency == RecurrencePatternFrequency.Weekly )
				{
					//	Uncheck all the checkboxes, then check the ones that now apply.
					this.chkSunday.Checked = false;
					this.chkMonday.Checked = false;
					this.chkTuesday.Checked = false;
					this.chkWednesday.Checked = false;
					this.chkThursday.Checked = false;
					this.chkFriday.Checked = false;
					this.chkSaturday.Checked = false;

					//	Check each textbox that has the corrseponding bit set for the
					//	day of the week it represents.
					RecurrencePatternDaysOfWeek daysOfWeek = (RecurrencePatternDaysOfWeek)value;
					if ( (daysOfWeek & RecurrencePatternDaysOfWeek.Sunday) == RecurrencePatternDaysOfWeek.Sunday )
						this.chkSunday.Checked = true;
					if ( (daysOfWeek & RecurrencePatternDaysOfWeek.Monday) == RecurrencePatternDaysOfWeek.Monday )
						this.chkMonday.Checked = true;
					if ( (daysOfWeek & RecurrencePatternDaysOfWeek.Tuesday) == RecurrencePatternDaysOfWeek.Tuesday )
						this.chkTuesday.Checked = true;
					if ( (daysOfWeek & RecurrencePatternDaysOfWeek.Wednesday) == RecurrencePatternDaysOfWeek.Wednesday )
						this.chkWednesday.Checked = true;
					if ( (daysOfWeek & RecurrencePatternDaysOfWeek.Thursday) == RecurrencePatternDaysOfWeek.Thursday )
						this.chkThursday.Checked = true;
					if ( (daysOfWeek & RecurrencePatternDaysOfWeek.Friday) == RecurrencePatternDaysOfWeek.Friday )
						this.chkFriday.Checked = true;
					if ( (daysOfWeek & RecurrencePatternDaysOfWeek.Saturday) == RecurrencePatternDaysOfWeek.Saturday )
						this.chkSaturday.Checked = true;
				}
				else
				if ( this.PatternFrequency == RecurrencePatternFrequency.Monthly ||
					 this.PatternFrequency == RecurrencePatternFrequency.Yearly )
				{
					string text = string.Empty;
					if ( value == RecurrencePatternDaysOfWeek.All ){ text = "day"; }
					else if ( value == RecurrencePatternDaysOfWeek.AllWeekdays ){ text = "weekday"; }
					else if ( value == RecurrencePatternDaysOfWeek.AllWeekendDays ){ text = "weekend day"; }
					else
						text = value.ToString();

					if ( this.PatternFrequency == RecurrencePatternFrequency.Monthly )
						this.cboDayOfWeekMonthly.Text = text;
					else
						this.cboDayOfWeekYearly.Text = text;
				}

			}
		}
			#endregion PatternDaysOfWeek

			#region OccurrenceStartTime
#if DEBUG
		/// <summary>
		/// Gets/sets the OccurrenceStartTime (cboOccurrenceStartTime)
		/// </summary>
#endif
		internal DateTime OccurrenceStartTime
		{
			get
			{ 
				DateTime startTime = DateTime.Parse( this.cboOccurrenceStartTime.Text );
				DateTime startDate = this.RangeStartDate;

				return new DateTime( startDate.Year, startDate.Month, startDate.Day,
									 startTime.Hour, startTime.Minute, 0 );
			}

			set
			{ 
				this.cboOccurrenceStartTime.Text = value.ToString( Utilities.TimeFormatString );
			}
		}
			#endregion OccurrenceStartTime

			#region OccurrenceDuration
#if DEBUG
		/// <summary>
		/// Gets/sets the OccurrenceDuration (cboOccurrenceDuration).
		/// </summary>
#endif
		internal TimeSpan OccurrenceDuration
		{
			get
			{ 
				TimeSpan retVal = TimeSpan.Zero;

				string itemText = this.cboOccurrenceDuration.Text;
				if ( /*this.cboOccurrenceDuration.IsDroppedDown &&*/
					 this.cboOccurrenceDuration.SelectedItem != null )
					itemText = this.cboOccurrenceDuration.SelectedItem.ToString();

				Utilities.ParseTimeSpan( itemText, out retVal );
				return retVal;
			}

			set
			{ 
				this.cboOccurrenceDuration.Text = Utilities.FormatTimeSpan( value, false ); 
			}
		}
			#endregion OccurrenceDuration

			#region PatternInterval
#if DEBUG
		/// <summary>
		/// Gets/sets the PatternInterval (txtPatternIntervalDaily, txtPatternIntervalWeekly, txtPatternIntervalExplicitMonthly)
		/// Not applicable when PatternFrequency = Yearly.
		/// </summary>
#endif
		internal int PatternInterval
		{
			get
			{ 
				string str = string.Empty;
				switch( this.PatternFrequency )
				{
					case RecurrencePatternFrequency.Daily:{ str = this.txtPatternIntervalDaily.Text; }break;
					case RecurrencePatternFrequency.Weekly:{ str = this.txtPatternIntervalWeekly.Text; }break;

					case RecurrencePatternFrequency.Monthly:
					{  
						str =	this.PatternType == RecurrencePatternType.Explicit ?
								this.txtPatternIntervalExplicitMonthly.Text :
								this.txtPatternIntervalCalculatedMonthly.Text;
					}
					break;
					
				}

				return Convert.ToInt32( str );
		
			}
			
			set
			{
				string newVal = value.ToString();

				switch( this.PatternFrequency )
				{
					case RecurrencePatternFrequency.Daily:
					{
						this.txtPatternIntervalDaily.Text = newVal;
					}
					break;

					case RecurrencePatternFrequency.Weekly:
					{ 
						this.txtPatternIntervalWeekly.Text = newVal;
					}
					break;

					case RecurrencePatternFrequency.Monthly:
					{ 
						//	Note that we set both here
						this.txtPatternIntervalExplicitMonthly.Text = newVal;
						this.txtPatternIntervalCalculatedMonthly.Text = newVal;
					}
					break;

				}
			}
		}
			#endregion PatternInterval

			#region PatternType
#if DEBUG
		/// <summary>
		/// Gets/sets the PatternType (optExplicitMonthly / optCalculatedMonthly, optExplicitYearly / optCalculatedYearly )
		/// Not applicable when PatternFrequency is Daily or Weekly.
		/// </summary>
#endif
		internal RecurrencePatternType PatternType
		{
			get
			{
				switch( this.PatternFrequency )
				{
					case RecurrencePatternFrequency.Daily:
					case RecurrencePatternFrequency.Weekly:
					{ 
					}
					break;

					case RecurrencePatternFrequency.Monthly:
					{
						return (RecurrencePatternType)this.optPatternType_Monthly.Value;
					}

					case RecurrencePatternFrequency.Yearly:
					{ 
						return (RecurrencePatternType)this.optPatternType_Yearly.Value;
					}
					
				}

				return RecurrenceDialog2007.defaultPatternType;
			}

			set
			{
				switch( this.PatternFrequency )
				{
					case RecurrencePatternFrequency.Daily:
					case RecurrencePatternFrequency.Weekly:
					{ 
					}
					break;

					case RecurrencePatternFrequency.Monthly:
					{
						this.optPatternType_Monthly.Value = value;
					}
					break;
					case RecurrencePatternFrequency.Yearly:
					{ 
						this.optPatternType_Yearly.Value = value;
					}
					break;
				}
			}
		}
			#endregion PatternType

			#region RangeMaxOccurrences
#if DEBUG
		/// <summary>
		/// Gets/sets the RangeMaxOccurrences (txtRangeMaxOccurrences)
		/// </summary>
#endif
		internal int RangeMaxOccurrences
		{
			get
			{
				return Convert.ToInt32( this.txtRangeMaxOccurrences.Text );
			}

			set
			{
				this.txtRangeMaxOccurrences.Text = value.ToString();
			}
		}
			#endregion RangeMaxOccurrences

			#region PatternDayOfMonth
#if DEBUG
		/// <summary>
		/// Gets/sets the PatternDayOfMonth (txtPatternDayOfMonthMonthly, txtPatternDayOfMonthYearly)
		/// Not applicable when PatternFrequency = Daily or Weekly.
		/// </summary>
#endif
		internal int PatternDayOfMonth
		{
			get
			{
				if ( this.PatternFrequency == RecurrencePatternFrequency.Monthly )
					return Convert.ToInt32( this.txtPatternDayOfMonthMonthly.Text );
				else
				if ( this.PatternFrequency == RecurrencePatternFrequency.Yearly )
					return Convert.ToInt32( this.txtPatternDayOfMonthYearly.Text );

				return RecurrenceDialog2007.defaultPatternDayOfMonth;
			}

			set
			{
				if ( this.PatternFrequency == RecurrencePatternFrequency.Monthly )
					this.txtPatternDayOfMonthMonthly.Text = value.ToString();
				else
				if ( this.PatternFrequency == RecurrencePatternFrequency.Yearly )
					this.txtPatternDayOfMonthYearly.Text = value.ToString();
			}
		}
			#endregion PatternDayOfMonth

			#region PatternOccurrenceOfDayInMonth
#if DEBUG
		/// <summary>
		/// Gets/sets the PatternOccurrenceOfDayInMonth (cboOccurrenceOfDayInMonthMonthly, cboOccurrenceOfDayInMonthYearly)
		/// </summary>
#endif
		internal RecurrencePatternOccurrenceOfDayInMonth PatternOccurrenceOfDayInMonth
		{
			get
			{
				if ( this.PatternFrequency == RecurrencePatternFrequency.Monthly )
				{
					ValueListItem item = this.cboOccurrenceOfDayInMonthMonthly.SelectedItem as ValueListItem;
					return item != null ? (RecurrencePatternOccurrenceOfDayInMonth)item.DataValue : RecurrencePatternOccurrenceOfDayInMonth.None;
				}
				else
				if ( this.PatternFrequency == RecurrencePatternFrequency.Yearly )
				{
					ValueListItem item = this.cboOccurrenceOfDayInMonthMonthly.SelectedItem as ValueListItem;
					return item != null ? (RecurrencePatternOccurrenceOfDayInMonth)item.DataValue : RecurrencePatternOccurrenceOfDayInMonth.None;
				}

				return RecurrencePatternOccurrenceOfDayInMonth.None;
			}

			set
			{
				int index = (int)value - 1;

				if ( this.PatternFrequency == RecurrencePatternFrequency.Monthly )
					this.cboOccurrenceOfDayInMonthMonthly.SelectedIndex = index;
				else
				if ( this.PatternFrequency == RecurrencePatternFrequency.Yearly )
					this.cboOccurrenceOfDayInMonthYearly.SelectedIndex = index;
			}
		}
			#endregion PatternOccurrenceOfDayInMonth

			#region PatternMonthOfYear
#if DEBUG
		/// <summary>
		/// Gets/sets the PatternMonthOfYear (cboMonthOfYearExplicitYearly, cboMonthOfYearCalculatedYearly)
		/// Only applicable when PatternFrequency = Yearly
		/// </summary>
#endif
		internal int PatternMonthOfYear
		{
			get
			{
				if ( this.PatternFrequency == RecurrencePatternFrequency.Yearly )
				{
					if ( this.PatternType == RecurrencePatternType.Explicit )
						return this.cboMonthOfYearExplicitYearly.SelectedIndex + 1;
					else
						return this.cboMonthOfYearCalculatedYearly.SelectedIndex + 1;
				}

				return RecurrenceDialog2007.defaultPatternMonthOfYear;
			}

			set
			{
				if ( this.PatternFrequency == RecurrencePatternFrequency.Yearly )
				{
					this.cboMonthOfYearExplicitYearly.SelectedIndex = value - 1;
					this.cboMonthOfYearCalculatedYearly.SelectedIndex = value - 1;;
				}
			}
		}
			#endregion PatternMonthOfYear

			#region ContentsChanged
#if DEBUG
		/// <summary>
		/// Returns whether any properties of this dialog's
		/// recurrence have changed since it was first displayed
		/// </summary>
#endif
		internal bool ContentsChanged
		{
			get
			{
				return	this.forceContentsChanged ||
						Utilities.AreRecurrencesEqual( this.initialAppointmentRecurrence, this.appointmentRecurrence ) == false;
			}
		}
			#endregion ContentsChanged

			#endregion Recurrence properties

			#region Static properties

			#region DateFormat
#if DEBUG
		/// <summary>
		/// Returns the format string to be used for dates.
		/// </summary>
#endif
		static internal string DateFormat
		{
			get{ return "ddd " + System.Globalization.CultureInfo.CurrentCulture.DateTimeFormat.ShortDatePattern;}
		}
			#endregion DateFormat

			#endregion Static properties

		#endregion Internal Properties

		#region Private / Internal Methods

			#region InitializeDialogControls
#if DEBUG
		/// <summary>
		/// Populates the ComboBoxes, assigns the appropriate localized string
		/// to each control's Text property.
		/// Also sets the dialog controls to their default values, which
		/// are based on the AppointmentRecurrence object this dialog is editing.
		/// </summary>
#endif
		private void InitializeDialogControls()
		{
			try
			{
				this.suspendOptionButtonHandling = true;

				//	Position the dark separator to be right next to the light one
				this.lblSepLight.Location = new Point( this.lblSepDark.Left + 1, this.lblSepDark.Top );

				//	Assign the recurrence to a stack variable
				AppointmentRecurrence recurrence = this.appointmentRecurrence;

				//	Populate the ComboBoxes
				this.PopulateStartTimeCombo();
				this.PopulateEndTimeCombo( recurrence.OccurrenceStartTime );
				this.PopulateDaysOfWeekCombos();
				this.PopulateMonthsOfYearCombos();
				this.PopulateOccurrenceOfDayInMonthCombos();
				this.PopulateDurationCombo();

				//	Populate the OptionSet ValueLists
				this.PopulateOptionSets();

				//	Set UseMnemonics for the OptionSets
				this.SetUseMnemonicsForOptionSets( this );

				//	Set each dialog control's value property to the appropriate
				//	value for the AppointRecurrence property it represents

				//	Appointment time properties
				this.OccurrenceStartTime = recurrence.OccurrenceStartTime;
				this.OccurrenceDuration = recurrence.OccurrenceDuration;
				this.cboEndTime.Text = recurrence.OccurrenceStartTime.Add( recurrence.OccurrenceDuration ).ToString( Utilities.TimeFormatString );

				//	Range properties
				this.RangeStartDate = recurrence.RangeStartDate;
				this.RangeEndDate = recurrence.RangeEndDate;
				this.RangeMaxOccurrences = recurrence.RangeMaxOccurrences;
				this.RangeLimit = recurrence.RangeLimit;

				//	Pattern properties
				this.PatternFrequency = recurrence.PatternFrequency;
				this.PatternInterval = recurrence.PatternInterval;
				this.PatternDaysOfWeek = recurrence.PatternDaysOfWeek;
				this.PatternDayOfMonth = recurrence.PatternDayOfMonth;
				this.PatternMonthOfYear = recurrence.PatternMonthOfYear;
				this.PatternOccurrenceOfDayInMonth = recurrence.PatternOccurrenceOfDayInMonth;
				this.PatternType = recurrence.PatternType;

				//	Compensate for the height of the missing option button
				//	if 'allowNoEndDate' is set to false.
				if ( this.allowNoEndDate == false )
				{
					this.optRangeLimit.Top += 20;
					this.optRangeLimit.Height -= 20;
				}
			}
			finally
			{
				this.suspendOptionButtonHandling = false;
			}
		}
			#endregion InitializeDialogControls

			#region HookDialogControlEvents
#if DEBUG
		/// <summary>
		/// Hooks/unhooks the events of interest for the various dialog controls.
		/// </summary>
		/// <param name="unhook">Specifies whether to hook or unhook the event handler.</param>
#endif
		private void HookDialogControlEvents( bool unhook )
		{
			this.HookCheckedChanged( this, unhook );
			this.HookOptionSetValueChanged( this, unhook );
			this.HookTextChanged( this, unhook );
			this.HookSelectionChangeCommitted( this, unhook );
			this.HookLeave( this, unhook );
			
			if ( ! unhook )
			{
				this.cboOccurrenceStartTime.SelectionChanged += new System.EventHandler(this.OccurrenceTimeComboBox_ItemChanged);
				this.cboOccurrenceStartTime.SelectionChangeCommitted += new System.EventHandler(this.OccurrenceTimeComboBox_ItemChanged);

				this.cboOccurrenceDuration.SelectionChanged += new System.EventHandler(this.OccurrenceTimeComboBox_ItemChanged);
				this.cboOccurrenceDuration.SelectionChangeCommitted += new System.EventHandler(this.OccurrenceTimeComboBox_ItemChanged);

				this.cboEndTime.SelectionChanged += new System.EventHandler(this.OccurrenceTimeComboBox_ItemChanged);
				this.cboEndTime.SelectionChangeCommitted += new System.EventHandler(this.OccurrenceTimeComboBox_ItemChanged);

			}
			else
			{
				this.cboOccurrenceStartTime.SelectionChanged -= new System.EventHandler(this.OccurrenceTimeComboBox_ItemChanged);
				this.cboOccurrenceStartTime.SelectionChangeCommitted -= new System.EventHandler(this.OccurrenceTimeComboBox_ItemChanged);

				this.cboOccurrenceDuration.SelectionChanged -= new System.EventHandler(this.OccurrenceTimeComboBox_ItemChanged);
				this.cboOccurrenceDuration.SelectionChangeCommitted -= new System.EventHandler(this.OccurrenceTimeComboBox_ItemChanged);

				this.cboEndTime.SelectionChanged -= new System.EventHandler(this.OccurrenceTimeComboBox_ItemChanged);
				this.cboEndTime.SelectionChangeCommitted -= new System.EventHandler(this.OccurrenceTimeComboBox_ItemChanged);

			}

		}
			#endregion HookDialogControlEvents

			#region HookCheckedChanged
#if DEBUG
		/// <summary>
		/// Hooks/unhooks the CheckedChanged event for CheckBoxes on the dialog.
		/// Note that this method calls itself recursively to hook the event for all descendants.
		/// </summary>
		/// <param name="parent">The control whose child controls will be included.</param>
		/// <param name="unhook">Specifies whether to hook or unhook the event handler.</param>
#endif
		private void HookCheckedChanged( Control parent, bool unhook )
		{
			if ( parent == null )
				return;

			if ( parent.Controls.Count > 0 )
			{
				foreach( Control child in parent.Controls )
				{
					UltraCheckEditor checkBox = child as UltraCheckEditor;
					if ( checkBox != null )
					{
						if ( unhook )
							checkBox.CheckedChanged -= new System.EventHandler(this.CheckBox_CheckedChanged);
						else
							checkBox.CheckedChanged += new System.EventHandler(this.CheckBox_CheckedChanged);
					}

					//	Call the method recursively on each child so that
					//	their children are set, etc.
					if ( child.Controls.Count > 0 )
						this.HookCheckedChanged( child, unhook );
				}
			}
		}
			#endregion HookCheckedChanged

			#region HookOptionSetValueChanged
#if DEBUG
		/// <summary>
		/// Hooks/unhooks the ValueChanged event for OptionSets on the dialog.
		/// Note that this method calls itself recursively to hook the event for all descendants.
		/// </summary>
		/// <param name="parent">The control whose child controls will be included.</param>
		/// <param name="unhook">Specifies whether to hook or unhook the event handler.</param>
#endif
		private void HookOptionSetValueChanged( Control parent, bool unhook )
		{
			if ( parent == null )
				return;

			if ( parent.Controls.Count > 0 )
			{
				foreach( Control child in parent.Controls )
				{
					UltraOptionSet option = child as UltraOptionSet;
					if ( option != null )
					{
						if ( unhook )
							option.ValueChanged -= new System.EventHandler(this.OptionSet_ValueChanged);
						else
							option.ValueChanged += new System.EventHandler(this.OptionSet_ValueChanged);
					}

					//	Call the method recursively on each child so that
					//	their children are set, etc.
					if ( child.Controls.Count > 0 )
						this.HookOptionSetValueChanged( child, unhook );
				}
			}
		}
			#endregion HookOptionSetValueChanged

			#region HookTextChanged
#if DEBUG
		/// <summary>
		/// Hooks/unhooks the CheckedChanged event for all text-based controls on the dialog.
		/// Note that this method calls itself recursively to hook the event for all descendants.
		/// </summary>
		/// <param name="parent">The control whose child controls will be included.</param>
		/// <param name="unhook">Specifies whether to hook or unhook the event handler.</param>
#endif
		private void HookTextChanged( Control parent, bool unhook )
		{
			if ( parent == null )
				return;

			if ( parent.Controls.Count > 0 )
			{
				foreach( Control child in parent.Controls )
				{
					if ( child is UltraTextEditor ||
						 child is UltraComboEditor ||
						 child is UltraCalendarCombo )
					{
						UltraComboEditor dropdownList = child as UltraComboEditor;
						if ( dropdownList != null && dropdownList.DropDownStyle == DropDownStyle.DropDownList )
							continue;

						if ( unhook )
							child.TextChanged -= new System.EventHandler(this.TextBasedControl_TextChanged);
						else
							child.TextChanged += new System.EventHandler(this.TextBasedControl_TextChanged);
					}

					//	Call the method recursively on each child so that
					//	their children are set, etc.
					if ( child.Controls.Count > 0 )
						this.HookTextChanged( child, unhook );
				}
			}
		}
			#endregion HookTextChanged

			#region HookSelectionChangeCommitted
#if DEBUG
		/// <summary>
		/// Hooks/unhooks the SelectionChangeCommitted event for all DropDownList-style ComboBox
		/// controls on the dialog. This was necessary because apparently DropDownList-style
		/// ComboBoxes do not fire TextChanged when the text has changed.
		/// Note that this method calls itself recursively to hook the event for all descendants.
		/// </summary>
		/// <param name="parent">The control whose child controls will be included.</param>
		/// <param name="unhook">Specifies whether to hook or unhook the event handler.</param>
#endif
		private void HookSelectionChangeCommitted( Control parent, bool unhook )
		{
			if ( parent == null )
				return;

			if ( parent.Controls.Count > 0 )
			{
				foreach( Control child in parent.Controls )
				{
					UltraComboEditor comboBox = child as UltraComboEditor;
					if ( comboBox != null && comboBox.DropDownStyle == DropDownStyle.DropDownList )
					{
						if ( unhook )
							comboBox.SelectionChangeCommitted -= new System.EventHandler(this.DropDownList_SelectionChangeCommitted);
						else
							comboBox.SelectionChangeCommitted += new System.EventHandler(this.DropDownList_SelectionChangeCommitted);
					}

					//	Call the method recursively on each child so that
					//	their children are set, etc.
					if ( child.Controls.Count > 0 )
						this.HookSelectionChangeCommitted( child, unhook );
				}
			}
		}
			#endregion HookSelectionChangeCommitted

			#region HookLeave
#if DEBUG
		/// <summary>
		/// Hooks/unhooks the Leave event for all edit controls on the dialog.
		/// Note that this method calls itself recursively to hook the event for all descendants.
		/// </summary>
		/// <param name="parent">The control whose child controls will be included.</param>
		/// <param name="unhook">Specifies whether to hook or unhook the event handler.</param>
#endif
		private void HookLeave( Control parent, bool unhook )
		{
			if ( parent == null )
				return;

			if ( parent.Controls.Count > 0 )
			{
				foreach( Control child in parent.Controls )
				{
					if ( child is UltraTextEditor ||
						 child is UltraCheckEditor ||
						 child is UltraComboEditor ||
						 child is UltraCalendarCombo )
					{
						if ( ! unhook )
							child.Leave += new EventHandler( this.OnLeaveDialogControl );
						else
							child.Leave -= new EventHandler( this.OnLeaveDialogControl );
					}

					//	Call the method recursively on each child so that
					//	their children are set, etc.
					if ( child.Controls.Count > 0 )
						this.HookLeave( child, unhook );
				}
			}
		}
				#endregion HookLeave

			#region HideAllPanels
#if DEBUG
		/// <summary>
		/// Hides all Panel controls.
		/// </summary>
#endif
		private void HideAllPanels()
		{
			this.pnlDaily.Visible = false;
			this.pnlWeekly.Visible = false;
			this.pnlMonthly.Visible = false;
			this.pnlYearly.Visible = false;
		}
			#endregion HideAllPanels

		#region SetUseMnemonicsForOptionSets
#if DEBUG
		/// <summary>
		/// Sets the UseMnemonics property for all option controls to true.
		/// </summary>
		/// <param name="parent">The control whose children will be enumerated.</param>
#endif
		private void SetUseMnemonicsForOptionSets( Control  parent )
		{
			if ( parent == null )
				return;

			if ( parent.Controls.Count > 0 )
			{
				foreach( Control child in parent.Controls )
				{
					UltraOptionSet optionSet = child as UltraOptionSet;
					if ( optionSet != null )
						optionSet.UseMnemonics = true;

					//	Call the method recursively on each child so that
					//	their children are set, etc.
					if ( child.Controls.Count > 0 )
						this.SetUseMnemonicsForOptionSets( child );
				}
			}
		}
			#endregion SetUseMnemonicsForOptionSets

			#region PopulateStartTimeCombo
#if DEBUG
		/// <summary>
		/// Populates the ComboBox that contains the values for the OccurrenceStartTime
		/// </summary>
#endif
		private void PopulateStartTimeCombo()
		{
			this.cboOccurrenceStartTime.Items.Clear();

			DateTime theDate = DateTime.Today.Date;
			for ( int i = 0; i < 48; i ++ )
			{
				this.cboOccurrenceStartTime.Items.Add( theDate.ToString(Utilities.TimeFormatString) );
				theDate = theDate.Add( new TimeSpan(0, 30, 0) );
			}

		}
			#endregion PopulateStartTimeCombo

			#region PopulateEndTimeCombo
#if DEBUG
		/// <summary>
		/// Populates the ComboBox that contains the values for the appointment's end time.
		/// </summary>
#endif
		private void PopulateEndTimeCombo( DateTime startTime )
		{
			this.cboEndTime.Items.Clear();

			//	To emulate MS Outlook, we will show the duration for each
			//	item only if the current duration is less than 1 day, and
			//	begin the run of items with 12AM, rather than the current
			//	start time.
			bool itemsShowDuration = (this.OccurrenceDuration.TotalDays < 1.0f);

			if ( ! itemsShowDuration )
				startTime = new DateTime( startTime.Year, startTime.Month, startTime.Day, 0, 0, 0 );

			for ( int i = 0; i < 48; i ++ )
			{
				TimeSpan timeSpan = new TimeSpan(0, i * 30, 0);
				DateTime theDate = startTime.Add( timeSpan );
				string itemText = theDate.ToString(Utilities.TimeFormatString) + (itemsShowDuration ? " " + Utilities.FormatTimeSpan( timeSpan, true ) : string.Empty);
				this.cboEndTime.Items.Add( itemText );
			}

		}
			#endregion PopulateEndTimeCombo

			#region PopulateMonthsOfYearCombos
#if DEBUG
		/// <summary>
		/// Populates the ComboBoxes that contain the values for the months of the year.
		/// </summary>
#endif
		private void PopulateMonthsOfYearCombos()
		{
			this.cboMonthOfYearExplicitYearly.Items.Clear();
			this.cboMonthOfYearCalculatedYearly.Items.Clear();

			string[] monthsOfYear = Utilities.MonthsOfYear;

			for ( int i = 0; i < monthsOfYear.GetLength(0); i ++ )
			{
				this.cboMonthOfYearExplicitYearly.Items.Add( monthsOfYear[i] );
				this.cboMonthOfYearCalculatedYearly.Items.Add( monthsOfYear[i] );
			}
		}
			#endregion PopulateMonthsOfYearCombos

			#region PopulateDaysOfWeekCombos
#if DEBUG
		/// <summary>
		/// Populates the ComboBoxes that contain the values for the months of the year.
		/// </summary>
#endif
		private void PopulateDaysOfWeekCombos()
		{
			this.cboDayOfWeekMonthly.Items.Clear();
			this.cboDayOfWeekYearly.Items.Clear();

			this.cboDayOfWeekMonthly.Items.Add( RecurrencePatternDaysOfWeek.All, "day" );
			this.cboDayOfWeekYearly.Items.Add( RecurrencePatternDaysOfWeek.All, "day" );

			this.cboDayOfWeekMonthly.Items.Add( RecurrencePatternDaysOfWeek.AllWeekdays, "weekday" );
			this.cboDayOfWeekYearly.Items.Add( RecurrencePatternDaysOfWeek.AllWeekdays, "weekday" );

			this.cboDayOfWeekMonthly.Items.Add( RecurrencePatternDaysOfWeek.AllWeekendDays, "weekend day" );
			this.cboDayOfWeekYearly.Items.Add( RecurrencePatternDaysOfWeek.AllWeekendDays, "weekend day" );

			for ( int i = (int)RecurrencePatternDaysOfWeek.Sunday;
				  i <= (int)RecurrencePatternDaysOfWeek.Saturday;
				  i = i * 2 )
			{
				RecurrencePatternDaysOfWeek daysOfWeek = (RecurrencePatternDaysOfWeek)i;

				this.cboDayOfWeekMonthly.Items.Add( daysOfWeek, daysOfWeek.ToString() );
				this.cboDayOfWeekYearly.Items.Add( daysOfWeek, daysOfWeek.ToString() );
			}

		}
			#endregion PopulateDaysOfWeekCombos

			#region PopulateOccurrenceOfDayInMonthCombos
#if DEBUG
		/// <summary>
		/// Populates the ComboBoxes that contain the values for the occurrence of the day in the month.
		/// </summary>
#endif
		private void PopulateOccurrenceOfDayInMonthCombos()
		{
			this.cboOccurrenceOfDayInMonthMonthly.Items.Clear();
			this.cboOccurrenceOfDayInMonthYearly.Items.Clear();

			this.cboOccurrenceOfDayInMonthMonthly.Items.Add( RecurrencePatternOccurrenceOfDayInMonth.First, "first" );
			this.cboOccurrenceOfDayInMonthYearly.Items.Add( RecurrencePatternOccurrenceOfDayInMonth.First, "first" );

			this.cboOccurrenceOfDayInMonthMonthly.Items.Add( RecurrencePatternOccurrenceOfDayInMonth.Second, "second" );
			this.cboOccurrenceOfDayInMonthYearly.Items.Add( RecurrencePatternOccurrenceOfDayInMonth.Second, "second" );

			this.cboOccurrenceOfDayInMonthMonthly.Items.Add( RecurrencePatternOccurrenceOfDayInMonth.Third, "third" );
			this.cboOccurrenceOfDayInMonthYearly.Items.Add( RecurrencePatternOccurrenceOfDayInMonth.Third, "third" );

			this.cboOccurrenceOfDayInMonthMonthly.Items.Add( RecurrencePatternOccurrenceOfDayInMonth.Fourth, "fourth" );
			this.cboOccurrenceOfDayInMonthYearly.Items.Add( RecurrencePatternOccurrenceOfDayInMonth.Fourth, "fourth" );

			this.cboOccurrenceOfDayInMonthMonthly.Items.Add( RecurrencePatternOccurrenceOfDayInMonth.Last, "last" );
			this.cboOccurrenceOfDayInMonthYearly.Items.Add( RecurrencePatternOccurrenceOfDayInMonth.Last, "last" );

		}
			#endregion PopulateOccurrenceOfDayInMonthCombos

			#region PopulateDurationCombo
#if DEBUG
		/// <summary>
		/// Populates the ComboBox that contain the values for the occurrence duration.
		/// </summary>
#endif
		private void PopulateDurationCombo()
		{
			this.cboOccurrenceDuration.Items.Clear();

			this.cboOccurrenceDuration.Items.Add( "0 minutes" );
			this.cboOccurrenceDuration.Items.Add( "5 minutes" );
			this.cboOccurrenceDuration.Items.Add( "10 minutes" );
			this.cboOccurrenceDuration.Items.Add( "15 minutes" );
			this.cboOccurrenceDuration.Items.Add( "30 minutes" );
			this.cboOccurrenceDuration.Items.Add( "1 hour" );
			this.cboOccurrenceDuration.Items.Add( "2 hours" );
			this.cboOccurrenceDuration.Items.Add( "3 hours" );
			this.cboOccurrenceDuration.Items.Add( "4 hours" );
			this.cboOccurrenceDuration.Items.Add( "5 hours" );
			this.cboOccurrenceDuration.Items.Add( "6 hours" );
			this.cboOccurrenceDuration.Items.Add( "7 hours" );
			this.cboOccurrenceDuration.Items.Add( "8 hours" );
			this.cboOccurrenceDuration.Items.Add( "9 hours" );
			this.cboOccurrenceDuration.Items.Add( "10 hours" );
			this.cboOccurrenceDuration.Items.Add( "11 hours" );
			this.cboOccurrenceDuration.Items.Add( ".5 days" );
			this.cboOccurrenceDuration.Items.Add( "18 hours" );
			this.cboOccurrenceDuration.Items.Add( "1 day" );
			this.cboOccurrenceDuration.Items.Add( "2 days" );
			this.cboOccurrenceDuration.Items.Add( "3 days" );
			this.cboOccurrenceDuration.Items.Add( "4 days" );
			this.cboOccurrenceDuration.Items.Add( "1 week" );
			this.cboOccurrenceDuration.Items.Add( "2 weeks" );

		}
			#endregion PopulateDurationCombo

			#region PopulateOptionSets
		private void PopulateOptionSets()
		{
			this.optPatternFrequency.Items.Clear();
			this.optPatternFrequency.Items.Add( RecurrencePatternFrequency.Daily, "&Daily" );
			this.optPatternFrequency.Items.Add( RecurrencePatternFrequency.Weekly, "&Weekly" );
			this.optPatternFrequency.Items.Add( RecurrencePatternFrequency.Monthly, "&Monthly" );
			this.optPatternFrequency.Items.Add( RecurrencePatternFrequency.Yearly, "&Yearly" );

			this.optRangeLimit.Items.Clear();

			if ( this.allowNoEndDate )
				this.optRangeLimit.Items.Add( RecurrenceRangeLimit.NoLimit, "N&o end date" );
			
			this.optRangeLimit.Items.Add( RecurrenceRangeLimit.LimitByNumberOfOccurrences, "End a&fter:" );
			this.optRangeLimit.Items.Add( RecurrenceRangeLimit.LimitByDate, "End &by:" );

			this.optPatternDaysOfWeek_Daily.Items.Clear();
			this.optPatternDaysOfWeek_Daily.Items.Add( RecurrencePatternDaysOfWeek.All, "E&very" );
			this.optPatternDaysOfWeek_Daily.Items.Add( RecurrencePatternDaysOfWeek.AllWeekdays, "Every wee&kday" );

			this.optPatternType_Monthly.Items.Clear();
			this.optPatternType_Monthly.Items.Add( RecurrencePatternType.Explicit, "D&ay" );
			this.optPatternType_Monthly.Items.Add( RecurrencePatternType.Calculated, "Th&e" );

			this.optPatternType_Yearly.Items.Clear();
			this.optPatternType_Yearly.Items.Add( RecurrencePatternType.Explicit, "E&very" );
			this.optPatternType_Yearly.Items.Add( RecurrencePatternType.Calculated, "Th&e" );

		}
			#endregion PopulateOptionSets

			#region ResetRecurrencePatternValues
#if DEBUG
		/// <summary>
		/// Restores the values of the controls on the specified Panel to their default values.
		/// </summary>
		/// <param name="patternFrequency">The PatternFrequency that identifies that Panel whose control values are to be reset.</param>
#endif
		private void ResetRecurrencePatternValues( RecurrencePatternFrequency patternFrequency )
		{
			try
			{
				//	Suspend updates while we are initializing the new panel's controls
				this.suspendUpdate = true;

				AppointmentRecurrence recurrence = this.appointmentRecurrence;
				switch ( patternFrequency )
				{
					case RecurrencePatternFrequency.Daily:
					case RecurrencePatternFrequency.Weekly:
					{
						this.PatternInterval = RecurrenceDialog2007.defaultPatternInterval;
						this.PatternDaysOfWeek =	(patternFrequency == RecurrencePatternFrequency.Daily) ?
													RecurrencePatternDaysOfWeek.All :
													Utilities.GetRecurrencePatternDaysOfWeekFromDayOfWeek( this.RangeStartDate.DayOfWeek );
					}
					break;

					case RecurrencePatternFrequency.Monthly:
					case RecurrencePatternFrequency.Yearly:
					{
						if ( patternFrequency == RecurrencePatternFrequency.Monthly )
							this.PatternInterval = RecurrenceDialog2007.defaultPatternInterval;
						else
							this.PatternMonthOfYear = this.RangeStartDate.Month;

						this.PatternDayOfMonth = this.RangeStartDate.Day;
						this.PatternOccurrenceOfDayInMonth = Utilities.GetOccurrenceOfDayInMonth( this.RangeStartDate );
						this.PatternDaysOfWeek = Utilities.GetRecurrencePatternDaysOfWeekFromDayOfWeek( this.RangeStartDate.DayOfWeek );
						this.PatternType = RecurrencePatternType.Explicit;
													
					}
					break;
				}
			}
			finally
			{
				//	Resume updates
				this.suspendUpdate = false;
			}

		}
			#endregion ResetRecurrencePatternValues

			#region UpdateRangeEndDate
#if DEBUG
		/// <summary>
		/// Updates the RangeEndDate to reflect the new default RangeEndDate
		/// based on the current pattern criteria.
		/// </summary>
#endif
		private void UpdateRangeEndDate()
		{
			if ( ! this.isInitializing && ! this.suspendUpdate )
				this.RangeEndDate = this.appointmentRecurrence.RangeEndDate;
		}
			#endregion UpdateRangeEndDate

			#region UpdateOccurrenceDuration
#if DEBUG
		/// <summary>
		/// Updates the value of the dialog's OccurrenceDuration field.
		/// </summary>
#endif
		private void UpdateOccurrenceDuration()
		{
			try
			{
				//	Get the integral number of days in the current duration
				int durationInDays = (int)this.OccurrenceDuration.TotalDays;

				//	Get the delta between the start and end time ComboBoxes
				TimeSpan delta = this.StartTimeEndTimeDelta;

				TimeSpan newDuration = new TimeSpan( 0, ( (durationInDays * 1440) + (int)(delta.TotalMinutes) ), 0 );

				//	If the new duration is negative, we take the difference between
				//	1 full day and that duration.
				if ( newDuration.TotalMinutes < 0 )
					newDuration = new TimeSpan( 0, (int)(1440.0 + newDuration.TotalMinutes), 0 );

				this.OccurrenceDuration = newDuration;

				this.appointmentRecurrence.OccurrenceDuration = this.OccurrenceDuration;
			}
			catch{}
		}
			#endregion UpdateOccurrenceDuration

			#region UpdateEndTime
#if DEBUG
		/// <summary>
		/// Updates the value of the dialog's cboEndTime field.
		/// </summary>
#endif
		private void UpdateEndTime()
		{
			try
			{
				//	Parse the value of the start time combo
				DateTime tempDate = DateTime.Parse( this.cboOccurrenceStartTime.Text );

				//	Repopulate the end time combo
				this.PopulateEndTimeCombo( tempDate );

				//	Set the end time combo's new text
				this.cboEndTime.Text = tempDate.Add( this.OccurrenceDuration ).ToString(Utilities.TimeFormatString);
			}
			catch{}
		}
			#endregion UpdateEndTime

			#region StartTimeEndTimeDelta
#if DEBUG
		/// <summary>
		/// Returns the amount of time between the parsed contents of
		/// the start and end time ComboBoxes. If the end time precedes
		/// the start time, a negative value is returned.
		/// </summary>
#endif
		private TimeSpan StartTimeEndTimeDelta
		{
			get
			{
				try
				{
					DateTime startTime = DateTime.Parse( this.cboOccurrenceStartTime.Text);
					string endTimeText = this.cboEndTime.Text;
					int pos = endTimeText.IndexOf( " (" );
					if ( pos >= 0 )
						endTimeText = endTimeText.Substring( 0, pos );

					DateTime endTime = DateTime.Parse( endTimeText);

					return endTime.Subtract( startTime );
				}
				catch
				{
					return TimeSpan.Zero;
				}
			}
		}
			#endregion StartTimeEndTimeDelta

			#region UpdateRecurrence
#if DEBUG
		/// <summary>
		/// Updates the applicable properties of the cached AppointmentRecurrence
		/// using values from the corresponding dialog fields. Note that we assume
		/// here that the applicable dialog fields contain valid values, since that is
		/// validated when each control is left.
		/// </summary>
#endif
		private void UpdateRecurrence()
		{
			if ( this.isInitializing || this.appointmentRecurrence == null )
				return;

			try
			{
				AppointmentRecurrence recurrence = this.appointmentRecurrence;

				RecurrencePatternFrequency patternFrequency = this.PatternFrequency;
				bool monthly = patternFrequency == RecurrencePatternFrequency.Monthly;
				bool yearly = patternFrequency == RecurrencePatternFrequency.Yearly;
				
				recurrence.PatternDaysOfWeek = this.PatternDaysOfWeek;
				recurrence.PatternFrequency = this.PatternFrequency;

				recurrence.OccurrenceStartTime = this.OccurrenceStartTime;
				recurrence.OccurrenceDuration = this.OccurrenceDuration;


				if ( monthly || yearly )
				{
					recurrence.PatternDayOfMonth = this.PatternDayOfMonth;
					recurrence.PatternOccurrenceOfDayInMonth = this.PatternOccurrenceOfDayInMonth;
					recurrence.PatternType = this.PatternType;
				}

				if ( ! yearly )
					recurrence.PatternInterval = this.PatternInterval;
				else
					recurrence.PatternMonthOfYear = this.PatternMonthOfYear;


				recurrence.RangeEndDate = this.RangeEndDate;
				recurrence.RangeLimit = this.RangeLimit;
				recurrence.RangeMaxOccurrences = this.RangeMaxOccurrences;
				recurrence.RangeStartDate = this.RangeStartDate;
			}
			catch
			{
			}
		}
			#endregion UpdateRecurrence

			#region VerifyCurrentDuration
#if DEBUG
		/// <summary>
		/// Verifies that the current value for the dialog's OccurrenceDuration
		/// property is valid and if it isn't, sets it to the maximum allowable
		/// value given the current PatternFrequency.
		/// </summary>
#endif
		private void VerifyCurrentDuration()
		{
			TimeSpan occurrenceDuration = this.OccurrenceDuration;
			double maxAllowableDays = 0.0f;
			switch ( this.PatternFrequency )
			{
				case RecurrencePatternFrequency.Daily:
					maxAllowableDays = 1.0f;
				break;

				case RecurrencePatternFrequency.Weekly:
					maxAllowableDays = 7.0f;
				break;

				case RecurrencePatternFrequency.Monthly:
					maxAllowableDays = 30.0f;
				break;

				case RecurrencePatternFrequency.Yearly:
					maxAllowableDays = 365.0f;
				break;
			}

			if ( occurrenceDuration.TotalDays > maxAllowableDays )
				this.OccurrenceDuration = new TimeSpan( (int)maxAllowableDays, 0, 0, 0 );
		}
			#endregion VerifyCurrentDuration

		#endregion Private / Internal Methods

		#region Control class overrides
		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				this.HookDialogControlEvents( true );

				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}
		#endregion Control class overrides

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			Infragistics.Win.ValueListItem valueListItem1 = new Infragistics.Win.ValueListItem();
			Infragistics.Win.ValueListItem valueListItem2 = new Infragistics.Win.ValueListItem();
			Infragistics.Win.ValueListItem valueListItem3 = new Infragistics.Win.ValueListItem();
			Infragistics.Win.ValueListItem valueListItem4 = new Infragistics.Win.ValueListItem();
			Infragistics.Win.UltraWinSchedule.CalendarCombo.DateButton dateButton1 = new Infragistics.Win.UltraWinSchedule.CalendarCombo.DateButton();
			Infragistics.Win.UltraWinSchedule.CalendarCombo.DateButton dateButton2 = new Infragistics.Win.UltraWinSchedule.CalendarCombo.DateButton();
			Infragistics.Win.ValueListItem valueListItem5 = new Infragistics.Win.ValueListItem();
			Infragistics.Win.ValueListItem valueListItem6 = new Infragistics.Win.ValueListItem();
			Infragistics.Win.ValueListItem valueListItem7 = new Infragistics.Win.ValueListItem();
			Infragistics.Win.ValueListItem valueListItem8 = new Infragistics.Win.ValueListItem();
			Infragistics.Win.ValueListItem valueListItem9 = new Infragistics.Win.ValueListItem();
			Infragistics.Win.ValueListItem valueListItem10 = new Infragistics.Win.ValueListItem();
			Infragistics.Win.ValueListItem valueListItem11 = new Infragistics.Win.ValueListItem();
			Infragistics.Win.ValueListItem valueListItem12 = new Infragistics.Win.ValueListItem();
			Infragistics.Win.ValueListItem valueListItem13 = new Infragistics.Win.ValueListItem();
			this.fraAppointmentTime = new Infragistics.Win.Misc.UltraGroupBox();
			this.cboOccurrenceDuration = new Infragistics.Win.UltraWinEditors.UltraComboEditor();
			this.lblOccurrenceDuration = new Infragistics.Win.Misc.UltraLabel();
			this.cboEndTime = new Infragistics.Win.UltraWinEditors.UltraComboEditor();
			this.lblEndTime = new Infragistics.Win.Misc.UltraLabel();
			this.cboOccurrenceStartTime = new Infragistics.Win.UltraWinEditors.UltraComboEditor();
			this.lblOccurrenceStartTime = new Infragistics.Win.Misc.UltraLabel();
			this.fraRecurrencePattern = new Infragistics.Win.Misc.UltraGroupBox();
			this.optPatternFrequency = new Infragistics.Win.UltraWinEditors.UltraOptionSet();
			this.lblSepDark = new Infragistics.Win.Misc.UltraLabel();
			this.lblSepLight = new Infragistics.Win.Misc.UltraLabel();
			this.fraRecurrenceRange = new Infragistics.Win.Misc.UltraGroupBox();
			this.dtpRangeEndDate = new Infragistics.Win.UltraWinSchedule.UltraCalendarCombo();
			this.lblRangeMaxOccurrences = new Infragistics.Win.Misc.UltraLabel();
			this.txtRangeMaxOccurrences = new Infragistics.Win.UltraWinEditors.UltraTextEditor();
			this.dtpRangeStartDate = new Infragistics.Win.UltraWinSchedule.UltraCalendarCombo();
			this.lblRangeStartDate = new Infragistics.Win.Misc.UltraLabel();
			this.optRangeLimit = new Infragistics.Win.UltraWinEditors.UltraOptionSet();
			this.cmdOk = new Infragistics.Win.Misc.UltraButton();
			this.cmdCancel = new Infragistics.Win.Misc.UltraButton();
			this.cmdRemoveRecurrence = new Infragistics.Win.Misc.UltraButton();
			this.pnlWeekly = new System.Windows.Forms.Panel();
			this.chkSaturday = new Infragistics.Win.UltraWinEditors.UltraCheckEditor();
			this.chkFriday = new Infragistics.Win.UltraWinEditors.UltraCheckEditor();
			this.chkThursday = new Infragistics.Win.UltraWinEditors.UltraCheckEditor();
			this.chkWednesday = new Infragistics.Win.UltraWinEditors.UltraCheckEditor();
			this.chkTuesday = new Infragistics.Win.UltraWinEditors.UltraCheckEditor();
			this.chkMonday = new Infragistics.Win.UltraWinEditors.UltraCheckEditor();
			this.chkSunday = new Infragistics.Win.UltraWinEditors.UltraCheckEditor();
			this.lblPatternInterval2Weekly = new Infragistics.Win.Misc.UltraLabel();
			this.txtPatternIntervalWeekly = new Infragistics.Win.UltraWinEditors.UltraTextEditor();
			this.lblPatternInterval1_Weekly = new Infragistics.Win.Misc.UltraLabel();
			this.pnlMonthly = new System.Windows.Forms.Panel();
			this.lblPatternInterval4Monthly = new Infragistics.Win.Misc.UltraLabel();
			this.txtPatternIntervalCalculatedMonthly = new Infragistics.Win.UltraWinEditors.UltraTextEditor();
			this.lblPatternInterval3Monthly = new Infragistics.Win.Misc.UltraLabel();
			this.cboDayOfWeekMonthly = new Infragistics.Win.UltraWinEditors.UltraComboEditor();
			this.cboOccurrenceOfDayInMonthMonthly = new Infragistics.Win.UltraWinEditors.UltraComboEditor();
			this.txtPatternIntervalExplicitMonthly = new Infragistics.Win.UltraWinEditors.UltraTextEditor();
			this.lblPatternInterval2Monthly = new Infragistics.Win.Misc.UltraLabel();
			this.lblPatternInterval1Monthly = new Infragistics.Win.Misc.UltraLabel();
			this.txtPatternDayOfMonthMonthly = new Infragistics.Win.UltraWinEditors.UltraTextEditor();
			this.optPatternType_Monthly = new Infragistics.Win.UltraWinEditors.UltraOptionSet();
			this.pnlDaily = new System.Windows.Forms.Panel();
			this.lblPatternIntervalDaily = new Infragistics.Win.Misc.UltraLabel();
			this.txtPatternIntervalDaily = new Infragistics.Win.UltraWinEditors.UltraTextEditor();
			this.optPatternDaysOfWeek_Daily = new Infragistics.Win.UltraWinEditors.UltraOptionSet();
			this.pnlYearly = new System.Windows.Forms.Panel();
			this.cboMonthOfYearCalculatedYearly = new Infragistics.Win.UltraWinEditors.UltraComboEditor();
			this.lblCalculatedYearly = new Infragistics.Win.Misc.UltraLabel();
			this.cboDayOfWeekYearly = new Infragistics.Win.UltraWinEditors.UltraComboEditor();
			this.cboOccurrenceOfDayInMonthYearly = new Infragistics.Win.UltraWinEditors.UltraComboEditor();
			this.txtPatternDayOfMonthYearly = new Infragistics.Win.UltraWinEditors.UltraTextEditor();
			this.cboMonthOfYearExplicitYearly = new Infragistics.Win.UltraWinEditors.UltraComboEditor();
			this.optPatternType_Yearly = new Infragistics.Win.UltraWinEditors.UltraOptionSet();
			((System.ComponentModel.ISupportInitialize)(this.fraAppointmentTime)).BeginInit();
			this.fraAppointmentTime.SuspendLayout();
			((System.ComponentModel.ISupportInitialize)(this.cboOccurrenceDuration)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.cboEndTime)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.cboOccurrenceStartTime)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.fraRecurrencePattern)).BeginInit();
			this.fraRecurrencePattern.SuspendLayout();
			((System.ComponentModel.ISupportInitialize)(this.optPatternFrequency)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.fraRecurrenceRange)).BeginInit();
			this.fraRecurrenceRange.SuspendLayout();
			((System.ComponentModel.ISupportInitialize)(this.dtpRangeEndDate)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.txtRangeMaxOccurrences)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.dtpRangeStartDate)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.optRangeLimit)).BeginInit();
			this.pnlWeekly.SuspendLayout();
			((System.ComponentModel.ISupportInitialize)(this.txtPatternIntervalWeekly)).BeginInit();
			this.pnlMonthly.SuspendLayout();
			((System.ComponentModel.ISupportInitialize)(this.txtPatternIntervalCalculatedMonthly)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.cboDayOfWeekMonthly)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.cboOccurrenceOfDayInMonthMonthly)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.txtPatternIntervalExplicitMonthly)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.txtPatternDayOfMonthMonthly)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.optPatternType_Monthly)).BeginInit();
			this.pnlDaily.SuspendLayout();
			((System.ComponentModel.ISupportInitialize)(this.txtPatternIntervalDaily)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.optPatternDaysOfWeek_Daily)).BeginInit();
			this.pnlYearly.SuspendLayout();
			((System.ComponentModel.ISupportInitialize)(this.cboMonthOfYearCalculatedYearly)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.cboDayOfWeekYearly)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.cboOccurrenceOfDayInMonthYearly)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.txtPatternDayOfMonthYearly)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.cboMonthOfYearExplicitYearly)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.optPatternType_Yearly)).BeginInit();
			this.SuspendLayout();
			// 
			// fraAppointmentTime
			// 
			this.fraAppointmentTime.Controls.AddRange(new System.Windows.Forms.Control[] {
																							 this.cboOccurrenceDuration,
																							 this.lblOccurrenceDuration,
																							 this.cboEndTime,
																							 this.lblEndTime,
																							 this.cboOccurrenceStartTime,
																							 this.lblOccurrenceStartTime});
			this.fraAppointmentTime.Location = new System.Drawing.Point(12, 8);
			this.fraAppointmentTime.Name = "fraAppointmentTime";
			this.fraAppointmentTime.Size = new System.Drawing.Size(438, 48);
			this.fraAppointmentTime.TabIndex = 0;
			this.fraAppointmentTime.Text = "Appointment time";
			// 
			// cboOccurrenceDuration
			// 
			this.cboOccurrenceDuration.Location = new System.Drawing.Point(336, 16);
			this.cboOccurrenceDuration.MaxDropDownItems = 7;
			this.cboOccurrenceDuration.Name = "cboOccurrenceDuration";
			this.cboOccurrenceDuration.Size = new System.Drawing.Size(92, 22);
			this.cboOccurrenceDuration.TabIndex = 5;
			// 
			// lblOccurrenceDuration
			// 
			this.lblOccurrenceDuration.AutoSize = true;
			this.lblOccurrenceDuration.Location = new System.Drawing.Point(284, 20);
			this.lblOccurrenceDuration.Name = "lblOccurrenceDuration";
			this.lblOccurrenceDuration.Size = new System.Drawing.Size(51, 15);
			this.lblOccurrenceDuration.TabIndex = 4;
			this.lblOccurrenceDuration.Text = "D&uration:";
			// 
			// cboEndTime
			// 
			this.cboEndTime.DropDownListWidth = 150;
			this.cboEndTime.Location = new System.Drawing.Point(176, 16);
			this.cboEndTime.MaxDropDownItems = 7;
			this.cboEndTime.Name = "cboEndTime";
			this.cboEndTime.Size = new System.Drawing.Size(92, 22);
			this.cboEndTime.TabIndex = 3;
			// 
			// lblEndTime
			// 
			this.lblEndTime.AutoSize = true;
			this.lblEndTime.Location = new System.Drawing.Point(144, 20);
			this.lblEndTime.Name = "lblEndTime";
			this.lblEndTime.Size = new System.Drawing.Size(23, 15);
			this.lblEndTime.TabIndex = 2;
			this.lblEndTime.Text = "E&nd";
			// 
			// cboOccurrenceStartTime
			// 
			this.cboOccurrenceStartTime.Location = new System.Drawing.Point(44, 16);
			this.cboOccurrenceStartTime.MaxDropDownItems = 7;
			this.cboOccurrenceStartTime.Name = "cboOccurrenceStartTime";
			this.cboOccurrenceStartTime.Size = new System.Drawing.Size(92, 22);
			this.cboOccurrenceStartTime.TabIndex = 1;
			// 
			// lblOccurrenceStartTime
			// 
			this.lblOccurrenceStartTime.AutoSize = true;
			this.lblOccurrenceStartTime.Location = new System.Drawing.Point(8, 20);
			this.lblOccurrenceStartTime.Name = "lblOccurrenceStartTime";
			this.lblOccurrenceStartTime.Size = new System.Drawing.Size(28, 15);
			this.lblOccurrenceStartTime.TabIndex = 0;
			this.lblOccurrenceStartTime.Text = "S&tart";
			// 
			// fraRecurrencePattern
			// 
			this.fraRecurrencePattern.Controls.AddRange(new System.Windows.Forms.Control[] {
																							   this.optPatternFrequency,
																							   this.lblSepDark,
																							   this.lblSepLight});
			this.fraRecurrencePattern.Location = new System.Drawing.Point(11, 58);
			this.fraRecurrencePattern.Name = "fraRecurrencePattern";
			this.fraRecurrencePattern.Size = new System.Drawing.Size(438, 116);
			this.fraRecurrencePattern.TabIndex = 1;
			this.fraRecurrencePattern.Text = "Recurrence pattern";
			// 
			// optPatternFrequency
			// 
			this.optPatternFrequency.BackColor = System.Drawing.Color.Transparent;
			this.optPatternFrequency.BorderStyle = Infragistics.Win.UIElementBorderStyle.None;
			valueListItem1.DataValue = "Daily";
			valueListItem2.DataValue = "ValueListItem1";
			valueListItem2.DisplayText = "Weekly";
			valueListItem3.DataValue = "ValueListItem2";
			valueListItem3.DisplayText = "Monthly";
			valueListItem4.DataValue = "ValueListItem3";
			valueListItem4.DisplayText = "Yearly";
			this.optPatternFrequency.Items.AddRange(new Infragistics.Win.ValueListItem[] {
																							 valueListItem1,
																							 valueListItem2,
																							 valueListItem3,
																							 valueListItem4});
			this.optPatternFrequency.ItemSpacingVertical = 4;
			this.optPatternFrequency.Location = new System.Drawing.Point(8, 24);
			this.optPatternFrequency.Name = "optPatternFrequency";
			this.optPatternFrequency.Size = new System.Drawing.Size(64, 80);
			this.optPatternFrequency.TabIndex = 12;
			// 
			// lblSepDark
			// 
			this.lblSepDark.BackColorInternal = System.Drawing.SystemColors.ControlDark;
			this.lblSepDark.Location = new System.Drawing.Point(72, 16);
			this.lblSepDark.Name = "lblSepDark";
			this.lblSepDark.Size = new System.Drawing.Size(1, 92);
			this.lblSepDark.TabIndex = 5;
			// 
			// lblSepLight
			// 
			this.lblSepLight.BackColorInternal = System.Drawing.SystemColors.ControlLightLight;
			this.lblSepLight.Location = new System.Drawing.Point(73, 16);
			this.lblSepLight.Name = "lblSepLight";
			this.lblSepLight.Size = new System.Drawing.Size(1, 92);
			this.lblSepLight.TabIndex = 4;
			// 
			// fraRecurrenceRange
			// 
			this.fraRecurrenceRange.Controls.AddRange(new System.Windows.Forms.Control[] {
																							 this.dtpRangeEndDate,
																							 this.lblRangeMaxOccurrences,
																							 this.txtRangeMaxOccurrences,
																							 this.dtpRangeStartDate,
																							 this.lblRangeStartDate,
																							 this.optRangeLimit});
			this.fraRecurrenceRange.Location = new System.Drawing.Point(11, 176);
			this.fraRecurrenceRange.Name = "fraRecurrenceRange";
			this.fraRecurrenceRange.Size = new System.Drawing.Size(438, 100);
			this.fraRecurrenceRange.TabIndex = 2;
			this.fraRecurrenceRange.Text = "Range of recurrence";
			// 
			// dtpRangeEndDate
			// 
			this.dtpRangeEndDate.BackColor = System.Drawing.SystemColors.Window;
			this.dtpRangeEndDate.DateButtons.Add(dateButton1);
			this.dtpRangeEndDate.Format = "ddd M/d/yyyy";
			this.dtpRangeEndDate.Location = new System.Drawing.Point(296, 67);
			this.dtpRangeEndDate.Name = "dtpRangeEndDate";
			this.dtpRangeEndDate.NonAutoSizeHeight = 21;
			this.dtpRangeEndDate.Size = new System.Drawing.Size(128, 21);
			this.dtpRangeEndDate.TabIndex = 7;
			// 
			// lblRangeMaxOccurrences
			// 
			this.lblRangeMaxOccurrences.AutoSize = true;
			this.lblRangeMaxOccurrences.Location = new System.Drawing.Point(336, 47);
			this.lblRangeMaxOccurrences.Name = "lblRangeMaxOccurrences";
			this.lblRangeMaxOccurrences.Size = new System.Drawing.Size(64, 15);
			this.lblRangeMaxOccurrences.TabIndex = 5;
			this.lblRangeMaxOccurrences.Text = "occurrences";
			// 
			// txtRangeMaxOccurrences
			// 
			this.txtRangeMaxOccurrences.Location = new System.Drawing.Point(296, 43);
			this.txtRangeMaxOccurrences.Name = "txtRangeMaxOccurrences";
			this.txtRangeMaxOccurrences.Size = new System.Drawing.Size(35, 22);
			this.txtRangeMaxOccurrences.TabIndex = 4;
			// 
			// dtpRangeStartDate
			// 
			this.dtpRangeStartDate.BackColor = System.Drawing.SystemColors.Window;
			this.dtpRangeStartDate.DateButtons.Add(dateButton2);
			this.dtpRangeStartDate.Format = "ddd M/d/yyyy";
			this.dtpRangeStartDate.Location = new System.Drawing.Point(48, 18);
			this.dtpRangeStartDate.Name = "dtpRangeStartDate";
			this.dtpRangeStartDate.NonAutoSizeHeight = 21;
			this.dtpRangeStartDate.Size = new System.Drawing.Size(152, 21);
			this.dtpRangeStartDate.TabIndex = 1;
			// 
			// lblRangeStartDate
			// 
			this.lblRangeStartDate.AutoSize = true;
			this.lblRangeStartDate.Location = new System.Drawing.Point(8, 20);
			this.lblRangeStartDate.Name = "lblRangeStartDate";
			this.lblRangeStartDate.Size = new System.Drawing.Size(32, 15);
			this.lblRangeStartDate.TabIndex = 0;
			this.lblRangeStartDate.Text = "&Start:";
			// 
			// optRangeLimit
			// 
			this.optRangeLimit.BackColor = System.Drawing.Color.Transparent;
			this.optRangeLimit.BorderStyle = Infragistics.Win.UIElementBorderStyle.None;
			valueListItem5.DataValue = "ValueListItem0";
			valueListItem5.DisplayText = "No End Date";
			valueListItem6.DataValue = "ValueListItem1";
			valueListItem6.DisplayText = "End after";
			valueListItem7.DataValue = "ValueListItem2";
			valueListItem7.DisplayText = "End by";
			this.optRangeLimit.Items.AddRange(new Infragistics.Win.ValueListItem[] {
																					   valueListItem5,
																					   valueListItem6,
																					   valueListItem7});
			this.optRangeLimit.ItemSpacingVertical = 6;
			this.optRangeLimit.Location = new System.Drawing.Point(208, 23);
			this.optRangeLimit.Name = "optRangeLimit";
			this.optRangeLimit.Size = new System.Drawing.Size(88, 64);
			this.optRangeLimit.TabIndex = 11;
			// 
			// cmdOk
			// 
			this.cmdOk.Location = new System.Drawing.Point(84, 284);
			this.cmdOk.Name = "cmdOk";
			this.cmdOk.TabIndex = 3;
			this.cmdOk.Text = "OK";
			this.cmdOk.Click += new System.EventHandler(this.cmdOk_Click);
			// 
			// cmdCancel
			// 
			this.cmdCancel.Location = new System.Drawing.Point(172, 284);
			this.cmdCancel.Name = "cmdCancel";
			this.cmdCancel.TabIndex = 4;
			this.cmdCancel.Text = "Cancel";
			this.cmdCancel.Click += new System.EventHandler(this.cmdCancel_Click);
			// 
			// cmdRemoveRecurrence
			// 
			this.cmdRemoveRecurrence.Location = new System.Drawing.Point(260, 284);
			this.cmdRemoveRecurrence.Name = "cmdRemoveRecurrence";
			this.cmdRemoveRecurrence.Size = new System.Drawing.Size(120, 23);
			this.cmdRemoveRecurrence.TabIndex = 5;
			this.cmdRemoveRecurrence.Text = "&Remove Recurrence";
			this.cmdRemoveRecurrence.Click += new System.EventHandler(this.cmdRemoveRecurrence_Click);
			// 
			// pnlWeekly
			// 
			this.pnlWeekly.Controls.AddRange(new System.Windows.Forms.Control[] {
																					this.chkSaturday,
																					this.chkFriday,
																					this.chkThursday,
																					this.chkWednesday,
																					this.chkTuesday,
																					this.chkMonday,
																					this.chkSunday,
																					this.lblPatternInterval2Weekly,
																					this.txtPatternIntervalWeekly,
																					this.lblPatternInterval1_Weekly});
			this.pnlWeekly.Location = new System.Drawing.Point(488, 116);
			this.pnlWeekly.Name = "pnlWeekly";
			this.pnlWeekly.Size = new System.Drawing.Size(350, 96);
			this.pnlWeekly.TabIndex = 7;
			this.pnlWeekly.Visible = false;
			// 
			// chkSaturday
			// 
			this.chkSaturday.Location = new System.Drawing.Point(176, 70);
			this.chkSaturday.Name = "chkSaturday";
			this.chkSaturday.Size = new System.Drawing.Size(72, 16);
			this.chkSaturday.TabIndex = 9;
			this.chkSaturday.Text = "Saturday";
			// 
			// chkFriday
			// 
			this.chkFriday.Location = new System.Drawing.Point(96, 70);
			this.chkFriday.Name = "chkFriday";
			this.chkFriday.Size = new System.Drawing.Size(72, 16);
			this.chkFriday.TabIndex = 8;
			this.chkFriday.Text = "Friday";
			// 
			// chkThursday
			// 
			this.chkThursday.Location = new System.Drawing.Point(16, 70);
			this.chkThursday.Name = "chkThursday";
			this.chkThursday.Size = new System.Drawing.Size(72, 16);
			this.chkThursday.TabIndex = 7;
			this.chkThursday.Text = "Thursday";
			// 
			// chkWednesday
			// 
			this.chkWednesday.Location = new System.Drawing.Point(253, 46);
			this.chkWednesday.Name = "chkWednesday";
			this.chkWednesday.Size = new System.Drawing.Size(88, 16);
			this.chkWednesday.TabIndex = 6;
			this.chkWednesday.Text = "Wednesday";
			// 
			// chkTuesday
			// 
			this.chkTuesday.Location = new System.Drawing.Point(176, 46);
			this.chkTuesday.Name = "chkTuesday";
			this.chkTuesday.Size = new System.Drawing.Size(72, 16);
			this.chkTuesday.TabIndex = 5;
			this.chkTuesday.Text = "Tuesday";
			// 
			// chkMonday
			// 
			this.chkMonday.Location = new System.Drawing.Point(96, 46);
			this.chkMonday.Name = "chkMonday";
			this.chkMonday.Size = new System.Drawing.Size(64, 16);
			this.chkMonday.TabIndex = 4;
			this.chkMonday.Text = "Monday";
			// 
			// chkSunday
			// 
			this.chkSunday.Location = new System.Drawing.Point(16, 46);
			this.chkSunday.Name = "chkSunday";
			this.chkSunday.Size = new System.Drawing.Size(64, 16);
			this.chkSunday.TabIndex = 3;
			this.chkSunday.Text = "Sunday";
			// 
			// lblPatternInterval2Weekly
			// 
			this.lblPatternInterval2Weekly.AutoSize = true;
			this.lblPatternInterval2Weekly.Location = new System.Drawing.Point(132, 12);
			this.lblPatternInterval2Weekly.Name = "lblPatternInterval2Weekly";
			this.lblPatternInterval2Weekly.Size = new System.Drawing.Size(64, 15);
			this.lblPatternInterval2Weekly.TabIndex = 2;
			this.lblPatternInterval2Weekly.Text = "week(s) on:";
			// 
			// txtPatternIntervalWeekly
			// 
			this.txtPatternIntervalWeekly.Location = new System.Drawing.Point(88, 10);
			this.txtPatternIntervalWeekly.Name = "txtPatternIntervalWeekly";
			this.txtPatternIntervalWeekly.Size = new System.Drawing.Size(35, 22);
			this.txtPatternIntervalWeekly.TabIndex = 1;
			// 
			// lblPatternInterval1_Weekly
			// 
			this.lblPatternInterval1_Weekly.AutoSize = true;
			this.lblPatternInterval1_Weekly.Location = new System.Drawing.Point(16, 12);
			this.lblPatternInterval1_Weekly.Name = "lblPatternInterval1_Weekly";
			this.lblPatternInterval1_Weekly.Size = new System.Drawing.Size(64, 15);
			this.lblPatternInterval1_Weekly.TabIndex = 0;
			this.lblPatternInterval1_Weekly.Text = "Re&cur every";
			// 
			// pnlMonthly
			// 
			this.pnlMonthly.Controls.AddRange(new System.Windows.Forms.Control[] {
																					 this.lblPatternInterval4Monthly,
																					 this.txtPatternIntervalCalculatedMonthly,
																					 this.lblPatternInterval3Monthly,
																					 this.cboDayOfWeekMonthly,
																					 this.cboOccurrenceOfDayInMonthMonthly,
																					 this.txtPatternIntervalExplicitMonthly,
																					 this.lblPatternInterval2Monthly,
																					 this.lblPatternInterval1Monthly,
																					 this.txtPatternDayOfMonthMonthly,
																					 this.optPatternType_Monthly});
			this.pnlMonthly.Location = new System.Drawing.Point(488, 220);
			this.pnlMonthly.Name = "pnlMonthly";
			this.pnlMonthly.Size = new System.Drawing.Size(350, 96);
			this.pnlMonthly.TabIndex = 8;
			this.pnlMonthly.Visible = false;
			// 
			// lblPatternInterval4Monthly
			// 
			this.lblPatternInterval4Monthly.AutoSize = true;
			this.lblPatternInterval4Monthly.Location = new System.Drawing.Point(296, 50);
			this.lblPatternInterval4Monthly.Name = "lblPatternInterval4Monthly";
			this.lblPatternInterval4Monthly.Size = new System.Drawing.Size(50, 15);
			this.lblPatternInterval4Monthly.TabIndex = 10;
			this.lblPatternInterval4Monthly.Text = "month(s)";
			// 
			// txtPatternIntervalCalculatedMonthly
			// 
			this.txtPatternIntervalCalculatedMonthly.Location = new System.Drawing.Point(260, 48);
			this.txtPatternIntervalCalculatedMonthly.Name = "txtPatternIntervalCalculatedMonthly";
			this.txtPatternIntervalCalculatedMonthly.Size = new System.Drawing.Size(35, 22);
			this.txtPatternIntervalCalculatedMonthly.TabIndex = 9;
			// 
			// lblPatternInterval3Monthly
			// 
			this.lblPatternInterval3Monthly.AutoSize = true;
			this.lblPatternInterval3Monthly.Location = new System.Drawing.Point(216, 51);
			this.lblPatternInterval3Monthly.Name = "lblPatternInterval3Monthly";
			this.lblPatternInterval3Monthly.Size = new System.Drawing.Size(45, 15);
			this.lblPatternInterval3Monthly.TabIndex = 8;
			this.lblPatternInterval3Monthly.Text = "of every";
			// 
			// cboDayOfWeekMonthly
			// 
			this.cboDayOfWeekMonthly.DropDownStyle = Infragistics.Win.DropDownStyle.DropDownList;
			this.cboDayOfWeekMonthly.Location = new System.Drawing.Point(130, 48);
			this.cboDayOfWeekMonthly.MaxDropDownItems = 10;
			this.cboDayOfWeekMonthly.Name = "cboDayOfWeekMonthly";
			this.cboDayOfWeekMonthly.Size = new System.Drawing.Size(86, 22);
			this.cboDayOfWeekMonthly.TabIndex = 7;
			// 
			// cboOccurrenceOfDayInMonthMonthly
			// 
			this.cboOccurrenceOfDayInMonthMonthly.DropDownStyle = Infragistics.Win.DropDownStyle.DropDownList;
			this.cboOccurrenceOfDayInMonthMonthly.Location = new System.Drawing.Point(58, 48);
			this.cboOccurrenceOfDayInMonthMonthly.Name = "cboOccurrenceOfDayInMonthMonthly";
			this.cboOccurrenceOfDayInMonthMonthly.Size = new System.Drawing.Size(70, 22);
			this.cboOccurrenceOfDayInMonthMonthly.TabIndex = 6;
			// 
			// txtPatternIntervalExplicitMonthly
			// 
			this.txtPatternIntervalExplicitMonthly.Location = new System.Drawing.Point(158, 14);
			this.txtPatternIntervalExplicitMonthly.Name = "txtPatternIntervalExplicitMonthly";
			this.txtPatternIntervalExplicitMonthly.Size = new System.Drawing.Size(35, 22);
			this.txtPatternIntervalExplicitMonthly.TabIndex = 3;
			// 
			// lblPatternInterval2Monthly
			// 
			this.lblPatternInterval2Monthly.AutoSize = true;
			this.lblPatternInterval2Monthly.Location = new System.Drawing.Point(206, 18);
			this.lblPatternInterval2Monthly.Name = "lblPatternInterval2Monthly";
			this.lblPatternInterval2Monthly.Size = new System.Drawing.Size(50, 15);
			this.lblPatternInterval2Monthly.TabIndex = 4;
			this.lblPatternInterval2Monthly.Text = "month(s)";
			// 
			// lblPatternInterval1Monthly
			// 
			this.lblPatternInterval1Monthly.AutoSize = true;
			this.lblPatternInterval1Monthly.Location = new System.Drawing.Point(106, 18);
			this.lblPatternInterval1Monthly.Name = "lblPatternInterval1Monthly";
			this.lblPatternInterval1Monthly.Size = new System.Drawing.Size(45, 15);
			this.lblPatternInterval1Monthly.TabIndex = 2;
			this.lblPatternInterval1Monthly.Text = "of every";
			// 
			// txtPatternDayOfMonthMonthly
			// 
			this.txtPatternDayOfMonthMonthly.Location = new System.Drawing.Point(58, 14);
			this.txtPatternDayOfMonthMonthly.Name = "txtPatternDayOfMonthMonthly";
			this.txtPatternDayOfMonthMonthly.Size = new System.Drawing.Size(35, 22);
			this.txtPatternDayOfMonthMonthly.TabIndex = 1;
			// 
			// optPatternType_Monthly
			// 
			this.optPatternType_Monthly.BackColor = System.Drawing.Color.Transparent;
			this.optPatternType_Monthly.BorderStyle = Infragistics.Win.UIElementBorderStyle.None;
			valueListItem8.DataValue = "ValueListItem0";
			valueListItem8.DisplayText = "Day";
			valueListItem9.DataValue = "ValueListItem1";
			valueListItem9.DisplayText = "The";
			this.optPatternType_Monthly.Items.AddRange(new Infragistics.Win.ValueListItem[] {
																								valueListItem8,
																								valueListItem9});
			this.optPatternType_Monthly.ItemSpacingVertical = 16;
			this.optPatternType_Monthly.Location = new System.Drawing.Point(16, 12);
			this.optPatternType_Monthly.Name = "optPatternType_Monthly";
			this.optPatternType_Monthly.Size = new System.Drawing.Size(40, 54);
			this.optPatternType_Monthly.TabIndex = 12;
			// 
			// pnlDaily
			// 
			this.pnlDaily.Controls.AddRange(new System.Windows.Forms.Control[] {
																				   this.lblPatternIntervalDaily,
																				   this.txtPatternIntervalDaily,
																				   this.optPatternDaysOfWeek_Daily});
			this.pnlDaily.Location = new System.Drawing.Point(488, 8);
			this.pnlDaily.Name = "pnlDaily";
			this.pnlDaily.Size = new System.Drawing.Size(350, 96);
			this.pnlDaily.TabIndex = 9;
			this.pnlDaily.Visible = false;
			// 
			// lblPatternIntervalDaily
			// 
			this.lblPatternIntervalDaily.AutoSize = true;
			this.lblPatternIntervalDaily.Location = new System.Drawing.Point(124, 20);
			this.lblPatternIntervalDaily.Name = "lblPatternIntervalDaily";
			this.lblPatternIntervalDaily.Size = new System.Drawing.Size(36, 15);
			this.lblPatternIntervalDaily.TabIndex = 2;
			this.lblPatternIntervalDaily.Text = "day(s)";
			// 
			// txtPatternIntervalDaily
			// 
			this.txtPatternIntervalDaily.Location = new System.Drawing.Point(80, 16);
			this.txtPatternIntervalDaily.Name = "txtPatternIntervalDaily";
			this.txtPatternIntervalDaily.Size = new System.Drawing.Size(35, 22);
			this.txtPatternIntervalDaily.TabIndex = 1;
			// 
			// optPatternDaysOfWeek_Daily
			// 
			this.optPatternDaysOfWeek_Daily.BackColor = System.Drawing.Color.Transparent;
			this.optPatternDaysOfWeek_Daily.BorderStyle = Infragistics.Win.UIElementBorderStyle.None;
			valueListItem10.DataValue = "ValueListItem0";
			valueListItem10.DisplayText = "Every";
			valueListItem11.DataValue = "ValueListItem1";
			valueListItem11.DisplayText = "Every weekday";
			this.optPatternDaysOfWeek_Daily.Items.AddRange(new Infragistics.Win.ValueListItem[] {
																									valueListItem10,
																									valueListItem11});
			this.optPatternDaysOfWeek_Daily.ItemSpacingVertical = 12;
			this.optPatternDaysOfWeek_Daily.Location = new System.Drawing.Point(16, 15);
			this.optPatternDaysOfWeek_Daily.Name = "optPatternDaysOfWeek_Daily";
			this.optPatternDaysOfWeek_Daily.Size = new System.Drawing.Size(96, 54);
			this.optPatternDaysOfWeek_Daily.TabIndex = 11;
			// 
			// pnlYearly
			// 
			this.pnlYearly.Controls.AddRange(new System.Windows.Forms.Control[] {
																					this.cboMonthOfYearCalculatedYearly,
																					this.lblCalculatedYearly,
																					this.cboDayOfWeekYearly,
																					this.cboOccurrenceOfDayInMonthYearly,
																					this.txtPatternDayOfMonthYearly,
																					this.cboMonthOfYearExplicitYearly,
																					this.optPatternType_Yearly});
			this.pnlYearly.Location = new System.Drawing.Point(488, 328);
			this.pnlYearly.Name = "pnlYearly";
			this.pnlYearly.Size = new System.Drawing.Size(350, 96);
			this.pnlYearly.TabIndex = 10;
			this.pnlYearly.Visible = false;
			// 
			// cboMonthOfYearCalculatedYearly
			// 
			this.cboMonthOfYearCalculatedYearly.DropDownStyle = Infragistics.Win.DropDownStyle.DropDownList;
			this.cboMonthOfYearCalculatedYearly.Location = new System.Drawing.Point(252, 48);
			this.cboMonthOfYearCalculatedYearly.MaxDropDownItems = 12;
			this.cboMonthOfYearCalculatedYearly.Name = "cboMonthOfYearCalculatedYearly";
			this.cboMonthOfYearCalculatedYearly.Size = new System.Drawing.Size(88, 22);
			this.cboMonthOfYearCalculatedYearly.TabIndex = 7;
			// 
			// lblCalculatedYearly
			// 
			this.lblCalculatedYearly.AutoSize = true;
			this.lblCalculatedYearly.Location = new System.Drawing.Point(231, 51);
			this.lblCalculatedYearly.Name = "lblCalculatedYearly";
			this.lblCalculatedYearly.Size = new System.Drawing.Size(14, 15);
			this.lblCalculatedYearly.TabIndex = 6;
			this.lblCalculatedYearly.Text = "of";
			// 
			// cboDayOfWeekYearly
			// 
			this.cboDayOfWeekYearly.DropDownStyle = Infragistics.Win.DropDownStyle.DropDownList;
			this.cboDayOfWeekYearly.Location = new System.Drawing.Point(142, 48);
			this.cboDayOfWeekYearly.MaxDropDownItems = 10;
			this.cboDayOfWeekYearly.Name = "cboDayOfWeekYearly";
			this.cboDayOfWeekYearly.Size = new System.Drawing.Size(86, 22);
			this.cboDayOfWeekYearly.TabIndex = 5;
			// 
			// cboOccurrenceOfDayInMonthYearly
			// 
			this.cboOccurrenceOfDayInMonthYearly.DropDownStyle = Infragistics.Win.DropDownStyle.DropDownList;
			this.cboOccurrenceOfDayInMonthYearly.Location = new System.Drawing.Point(70, 48);
			this.cboOccurrenceOfDayInMonthYearly.Name = "cboOccurrenceOfDayInMonthYearly";
			this.cboOccurrenceOfDayInMonthYearly.Size = new System.Drawing.Size(70, 22);
			this.cboOccurrenceOfDayInMonthYearly.TabIndex = 4;
			// 
			// txtPatternDayOfMonthYearly
			// 
			this.txtPatternDayOfMonthYearly.Location = new System.Drawing.Point(160, 14);
			this.txtPatternDayOfMonthYearly.Name = "txtPatternDayOfMonthYearly";
			this.txtPatternDayOfMonthYearly.Size = new System.Drawing.Size(35, 22);
			this.txtPatternDayOfMonthYearly.TabIndex = 2;
			// 
			// cboMonthOfYearExplicitYearly
			// 
			this.cboMonthOfYearExplicitYearly.DropDownStyle = Infragistics.Win.DropDownStyle.DropDownList;
			this.cboMonthOfYearExplicitYearly.Location = new System.Drawing.Point(70, 14);
			this.cboMonthOfYearExplicitYearly.MaxDropDownItems = 12;
			this.cboMonthOfYearExplicitYearly.Name = "cboMonthOfYearExplicitYearly";
			this.cboMonthOfYearExplicitYearly.Size = new System.Drawing.Size(88, 22);
			this.cboMonthOfYearExplicitYearly.TabIndex = 1;
			// 
			// optPatternType_Yearly
			// 
			this.optPatternType_Yearly.BackColor = System.Drawing.Color.Transparent;
			this.optPatternType_Yearly.BorderStyle = Infragistics.Win.UIElementBorderStyle.None;
			valueListItem12.DataValue = "ValueListItem0";
			valueListItem12.DisplayText = "Every";
			valueListItem13.DataValue = "ValueListItem1";
			valueListItem13.DisplayText = "The";
			this.optPatternType_Yearly.Items.AddRange(new Infragistics.Win.ValueListItem[] {
																							   valueListItem12,
																							   valueListItem13});
			this.optPatternType_Yearly.ItemSpacingVertical = 18;
			this.optPatternType_Yearly.Location = new System.Drawing.Point(8, 10);
			this.optPatternType_Yearly.Name = "optPatternType_Yearly";
			this.optPatternType_Yearly.Size = new System.Drawing.Size(56, 70);
			this.optPatternType_Yearly.TabIndex = 13;
			// 
			// RecurrenceDialog2007
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 14);
			this.ClientSize = new System.Drawing.Size(848, 486);
			this.Controls.AddRange(new System.Windows.Forms.Control[] {
																		  this.pnlYearly,
																		  this.pnlDaily,
																		  this.pnlMonthly,
																		  this.pnlWeekly,
																		  this.cmdRemoveRecurrence,
																		  this.cmdCancel,
																		  this.fraRecurrenceRange,
																		  this.fraRecurrencePattern,
																		  this.fraAppointmentTime,
																		  this.cmdOk});
			this.Font = new System.Drawing.Font("Tahoma", 8.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.Name = "RecurrenceDialog2007";
			this.ShowInTaskbar = false;
			this.Text = "Appointment Recurrence";
			((System.ComponentModel.ISupportInitialize)(this.fraAppointmentTime)).EndInit();
			this.fraAppointmentTime.ResumeLayout(false);
			((System.ComponentModel.ISupportInitialize)(this.cboOccurrenceDuration)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.cboEndTime)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.cboOccurrenceStartTime)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.fraRecurrencePattern)).EndInit();
			this.fraRecurrencePattern.ResumeLayout(false);
			((System.ComponentModel.ISupportInitialize)(this.optPatternFrequency)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.fraRecurrenceRange)).EndInit();
			this.fraRecurrenceRange.ResumeLayout(false);
			((System.ComponentModel.ISupportInitialize)(this.dtpRangeEndDate)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.txtRangeMaxOccurrences)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.dtpRangeStartDate)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.optRangeLimit)).EndInit();
			this.pnlWeekly.ResumeLayout(false);
			((System.ComponentModel.ISupportInitialize)(this.txtPatternIntervalWeekly)).EndInit();
			this.pnlMonthly.ResumeLayout(false);
			((System.ComponentModel.ISupportInitialize)(this.txtPatternIntervalCalculatedMonthly)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.cboDayOfWeekMonthly)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.cboOccurrenceOfDayInMonthMonthly)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.txtPatternIntervalExplicitMonthly)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.txtPatternDayOfMonthMonthly)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.optPatternType_Monthly)).EndInit();
			this.pnlDaily.ResumeLayout(false);
			((System.ComponentModel.ISupportInitialize)(this.txtPatternIntervalDaily)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.optPatternDaysOfWeek_Daily)).EndInit();
			this.pnlYearly.ResumeLayout(false);
			((System.ComponentModel.ISupportInitialize)(this.cboMonthOfYearCalculatedYearly)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.cboDayOfWeekYearly)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.cboOccurrenceOfDayInMonthYearly)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.txtPatternDayOfMonthYearly)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.cboMonthOfYearExplicitYearly)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.optPatternType_Yearly)).EndInit();
			this.ResumeLayout(false);

		}
		#endregion Windows Form Designer generated code

		#region Event handlers

			#region DropDownList_SelectionChangeCommitted
#if DEBUG
		/// <summary>
		/// Handles the SelectionChangeCommitted event for the DropDownList-style ComboBoxes.
		/// </summary>
		/// <param name="sender">The ComboBox whose selection change was committed.</param>
		/// <param name="e">Event arguments</param>
#endif
		private void DropDownList_SelectionChangeCommitted(object sender, System.EventArgs e)
		{
			if ( this.suspendEventHandling )
				return;

			UltraComboEditor comboBox = sender as UltraComboEditor;
			if ( comboBox != null )
			{
				if ( comboBox == this.cboOccurrenceOfDayInMonthMonthly ||
					 comboBox == this.cboDayOfWeekMonthly )
					this.optPatternType_Monthly.Value = RecurrencePatternType.Calculated;
				else
				if ( comboBox == this.cboMonthOfYearExplicitYearly )
					this.optPatternType_Yearly.Value = RecurrencePatternType.Explicit;
				else
				if ( comboBox == this.cboOccurrenceOfDayInMonthYearly ||
					 comboBox == this.cboDayOfWeekYearly ||
					 comboBox == this.cboMonthOfYearCalculatedYearly )
					this.optPatternType_Yearly.Value = RecurrencePatternType.Calculated;
			}
		}
			#endregion DropDownList_SelectionChangeCommitted

			#region TextBasedControl_TextChanged
#if DEBUG
		/// <summary>
		/// Handles the TextChanged event for the text-based controls.
		/// </summary>
		/// <param name="sender">The control whose text has changed.</param>
		/// <param name="e">Event arguments</param>
#endif
		private void TextBasedControl_TextChanged(object sender, System.EventArgs e)
		{
			if ( this.suspendEventHandling )
				return;

			UltraTextEditor textBox = sender as UltraTextEditor;
			UltraCalendarCombo dateTimePicker = sender as UltraCalendarCombo;

			if ( textBox != null )
			{
				if ( textBox == this.txtPatternIntervalDaily )
					this.optPatternDaysOfWeek_Daily.Value = RecurrencePatternDaysOfWeek.All;
				else
				if ( textBox == this.txtPatternIntervalWeekly )
					this.UpdateRangeEndDate();
				else
				if ( textBox == this.txtPatternDayOfMonthMonthly ||
					 textBox == this.txtPatternIntervalExplicitMonthly )
					this.optPatternType_Monthly.Value = RecurrencePatternType.Explicit;
				else
				if ( textBox == this.txtPatternIntervalCalculatedMonthly )
					this.optPatternType_Monthly.Value = RecurrencePatternType.Calculated;
				else
				if ( textBox == this.txtPatternDayOfMonthYearly )
					this.optPatternType_Yearly.Value = RecurrencePatternType.Explicit;
				else
				if ( textBox == this.txtRangeMaxOccurrences )
					this.optRangeLimit.Value = RecurrenceRangeLimit.LimitByNumberOfOccurrences;
			}
			else
			if ( dateTimePicker != null )
			{
				if ( dateTimePicker == this.dtpRangeStartDate )
					this.UpdateRangeEndDate();
				else
				if ( dateTimePicker == this.dtpRangeEndDate )
				{
					//	Only do this when the DateTimePicker has focus
					if ( dateTimePicker.Controls.Count > 0 &&
						 this.ActiveControl == dateTimePicker.Controls[0] )
						this.optRangeLimit.Value = RecurrenceRangeLimit.LimitByDate;
				}
			}

		}
			#endregion TextBasedControl_TextChanged

			#region CheckBox_CheckedChanged
#if DEBUG
		/// <summary>
		/// Handles the CheckChanged event for the checkboxes
		/// that represent the days of the week when PatternFrequency
		/// is set to Weekly.
		/// </summary>
		/// <param name="sender">The CheckBox that was clicked.</param>
		/// <param name="e">Event arguments</param>
#endif
		private void CheckBox_CheckedChanged(object sender, System.EventArgs e)
		{
			if ( this.suspendEventHandling )
				return;

			this.OnValueChangeCommitted( sender as Control, PropertyIds.PatternDaysOfWeek );
		}
			#endregion CheckBox_CheckedChanged


			#region OptionSet_ValueChanged
#if DEBUG
		/// <summary>
		/// Handles the ValueChanged event for the option set controls
		/// that represent the pattern-related properties.
		/// </summary>
		/// <param name="sender">The option set that was clicked.</param>
		/// <param name="e">Event arguments</param>
#endif
		private void OptionSet_ValueChanged(object sender, System.EventArgs e)
		{
			if ( this.suspendEventHandling )
				return;

			UltraOptionSet option = sender as UltraOptionSet;

			if ( this.suspendOptionButtonHandling && option != this.optPatternFrequency )
				return;

			PropertyIds propId = PropertyIds.Recurrence;

			if ( option == this.optPatternFrequency )
			{
				this.PatternFrequencyOptionSet_ValueChanged( option );
				propId = PropertyIds.PatternFrequency;
			}
			else
			if ( option == this.optRangeLimit )
			{
				this.RangeOptionSet_ValueChanged( option );
				propId = PropertyIds.RangeLimit;
			}
			else
			if ( option == this.optPatternDaysOfWeek_Daily )
			{
				if ( (RecurrencePatternDaysOfWeek)this.optPatternDaysOfWeek_Daily.Value == RecurrencePatternDaysOfWeek.AllWeekdays )
					this.PatternInterval = RecurrenceDialog2007.defaultPatternInterval;

				propId = PropertyIds.PatternDaysOfWeek;
			}
			else
			if ( option == this.optPatternType_Monthly ||
				 option == this.optPatternType_Yearly )
			{
				if ( option == this.optPatternType_Monthly )
					this.PatternInterval = RecurrenceDialog2007.defaultPatternInterval;
				else
					this.PatternMonthOfYear = this.RangeStartDate.Month;

				this.PatternDayOfMonth = this.RangeStartDate.Day;
				this.PatternOccurrenceOfDayInMonth = Utilities.GetOccurrenceOfDayInMonth( this.RangeStartDate );
				this.PatternDaysOfWeek = Utilities.GetRecurrencePatternDaysOfWeekFromDayOfWeek( this.RangeStartDate.DayOfWeek );

				propId = PropertyIds.PatternType;
			}
			else
				Debug.Assert( false, "Finish this" );

			if ( option != null && option.Value != null )
				this.OnValueChangeCommitted( option, propId );
		}
			#endregion OptionSet_ValueChanged

			#region RangeOptionSet_ValueChanged
#if DEBUG
		/// <summary>
		/// Handles the ValueChanged event for the option buttons
		/// that represent the range-related properties.
		/// </summary>
		/// <param name="option">The option set that was clicked.</param>
#endif
		private void RangeOptionSet_ValueChanged( UltraOptionSet option )
		{
			if ( this.suspendEventHandling )
				return;

			if ( option != null && option.Value is RecurrenceRangeLimit )
			{
				if ( (RecurrenceRangeLimit)option.Value == RecurrenceRangeLimit.NoLimit )
				{
					try
					{
						this.suspendEventHandling = true;
						this.RangeMaxOccurrences = RecurrenceDialog2007.defaultRangeMaxOccurrences;
					}
					finally
					{
						this.suspendEventHandling = false;
					}
				}
			}
		}
			#endregion RangeOptionSet_ValueChanged

		#region PatternFrequencyOptionSet_ValueChanged
#if DEBUG
		/// <summary>
		/// Handles the ValueChanged event for the option buttons
		/// that represent the different values for PatternFrequency.
		/// </summary>
		/// <param name="option">The option that was clicked.</param>
#endif
		private void PatternFrequencyOptionSet_ValueChanged( UltraOptionSet option )
		{
			if ( this.suspendEventHandling )
				return;

			if ( option != null && option.Value is RecurrencePatternFrequency )
			{
				//	Ensure all the other panels are hidden
				this.HideAllPanels();
				Panel activePanel = null;

				//	Set the new panel based on which option was clicked
				if ( (RecurrencePatternFrequency)option.Value == RecurrencePatternFrequency.Daily )
					activePanel = this.pnlDaily;
				else
				if ( (RecurrencePatternFrequency)option.Value == RecurrencePatternFrequency.Weekly )
					activePanel = this.pnlWeekly;
				else
				if ( (RecurrencePatternFrequency)option.Value == RecurrencePatternFrequency.Monthly )
					activePanel = this.pnlMonthly;
				else
				if ( (RecurrencePatternFrequency)option.Value == RecurrencePatternFrequency.Yearly )
					activePanel = this.pnlYearly;

				if ( activePanel != null )
				{
					//	Set the size and location of the panel
					activePanel.Location = this.PanelLocation;
					activePanel.Size = this.PanelSize;

					//	Set the parent to the 'Recurrence Pattern' frame
					activePanel.Parent = this.fraRecurrencePattern;

					//	Adjust the panel's TabIndex so it immediately follows
					//	the option to which it corresponds.
					activePanel.TabIndex = option.TabIndex + 1;

					//	Reset the control values to their defaults each time
					//	a new panel is displayed.
					this.ResetRecurrencePatternValues( this.PatternFrequency );

					//	We will deviate from MS Outlook here...when a new
					//	PatternFrequency is selected, if the existing occurrence
					//	duration is too big, we will reduce it to the maximum
					//	allowable value for that PatternFrequency.
					this.VerifyCurrentDuration();

					//	Since we supress updates when showing a new panel, update
					//	the range end date now that the panel is up to date
					this.UpdateRecurrence();
					this.UpdateRangeEndDate();

					//	Show the panel
					activePanel.Visible = true;

				}
			}
		}
			#endregion PatternFrequencyOptionSet_ValueChanged

			#region OccurrenceTimeComboBox_ItemChanged
		private void OccurrenceTimeComboBox_ItemChanged(object sender, System.EventArgs e)
		{
			if ( this.suspendEventHandling || this.isInitializing )
				return;

			//	Invoke the OnAfterEndTimeSelected method so we can strip
			//	the duration out of the item text so we only see the time
			//	in the edit portion.
			if ( sender == this.cboEndTime )
				this.BeginInvoke( new MethodInvoker(new afterSelection_cboEndTime( this.OnAfterEndTimeSelected ) ) );
			else
			if ( sender == this.cboOccurrenceDuration ||
				 sender == this.cboOccurrenceStartTime )
				this.UpdateEndTime();
		}

		private void OnAfterEndTimeSelected()
		{
			UltraComboEditor comboBox = this.cboEndTime;
			if ( comboBox.SelectedIndex != -1 )
			{
				string text = comboBox.SelectedItem.DisplayText as string;
				if ( text == comboBox.Text )
				{
					int end = text.IndexOf( " (" );
					if ( end >= 0 )
						text = text.Substring(0, end );

					comboBox.Text = text;
					
					this.UpdateOccurrenceDuration();
				}
			}
		}
			#endregion cboEndTime_ItemChanged

			#region OnLeaveDialogControl
#if DEBUG
		/// <summary>
		/// Handles the Leave event for all text-based edit controls on the form.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
#endif
		private void OnLeaveDialogControl( object sender, EventArgs e )
		{
			if ( this.suspendEventHandling )
				return;

			if ( sender == this.cboOccurrenceDuration )
				this.OnValueChangeCommitted( sender as Control, PropertyIds.OccurrenceDuration );
			else
			if ( sender == this.cboOccurrenceStartTime )
				this.OnValueChangeCommitted( sender as Control, PropertyIds.OccurrenceStartTime );
			else
			if ( sender == this.cboEndTime )
				//	Note that since there is no 'OccurrenceEndTime' property,
				//	we will notify as a change in OccurrenceDuration, since
				//	that is the thing we need to reevaluate.
				this.OnValueChangeCommitted( sender as Control, PropertyIds.OccurrenceDuration );
			else
			if ( sender == this.txtPatternDayOfMonthMonthly ||
				 sender == this.txtPatternDayOfMonthYearly )
				this.OnValueChangeCommitted( sender as Control, PropertyIds.PatternDayOfMonth );
			else
			if ( sender == this.cboDayOfWeekMonthly ||
				 sender == this.cboDayOfWeekYearly )
				this.OnValueChangeCommitted( sender as Control, PropertyIds.PatternDaysOfWeek );
			else
			if ( sender == this.txtPatternIntervalDaily ||
				 sender == this.txtPatternIntervalWeekly ||
				 sender == this.txtPatternIntervalExplicitMonthly ||
				 sender == this.txtPatternIntervalCalculatedMonthly )
				this.OnValueChangeCommitted( sender as Control, PropertyIds.PatternInterval );
			else
			if ( sender == this.cboMonthOfYearExplicitYearly ||
				 sender == this.cboMonthOfYearCalculatedYearly )
				this.OnValueChangeCommitted( sender as Control, PropertyIds.PatternMonthOfYear );
			else
			if ( sender == this.cboOccurrenceOfDayInMonthMonthly ||
				 sender == this.cboOccurrenceOfDayInMonthYearly )
				this.OnValueChangeCommitted( sender as Control, PropertyIds.PatternOccurrenceOfDayInMonth );
			else
			if ( sender == this.dtpRangeEndDate )
				this.OnValueChangeCommitted( sender as Control, PropertyIds.RangeEndDate );
			else
			if ( sender == this.txtRangeMaxOccurrences )
				this.OnValueChangeCommitted( sender as Control, PropertyIds.RangeMaxOccurrences );
			else
			if ( sender == this.dtpRangeStartDate )
				this.OnValueChangeCommitted( sender as Control, PropertyIds.RangeStartDate );

		}

#if DEBUG
		/// <summary>
		/// Called when a change to a dialog control has been committed, i.e., the control was left.
		/// This method is used to validate user input; when each dialog control is left, the user's
		/// entry is converted to the appropriate data type for the corresponding AppointmentRecurrence
		/// property, and then that property is set on the temporary AppointmentRecurrence object we use
		/// for validation. If the input is valid, the RangeEndDate is updated; if it isn't, the error
		/// dialog is displayed, and the control is focused and its text selected.
		/// </summary>
		/// <param name="sender">The control that was left.</param>
		/// <param name="recurrencePropId">The property identifier of the AppointmentRecurrence property that has logically changed.</param>
#endif
		private void OnValueChangeCommitted( Control sender, PropertyIds recurrencePropId )
		{
			if ( sender == null )
				return;

			if ( this.suspendUpdate || this.isInitializing )
				return;

			this.contentsAreValid = true;

			//	We wrap this in a try/catch because the attempt to convert
			//	the dialog field's value could fail, and also, the attempt
			//	to set the property on the AppointmentRecurrence could fail.
			bool displayGenericError = true;
			try
			{
				#region StartTime / EndTime / Duration ComboBoxes
				if ( sender == this.cboOccurrenceStartTime ||
					 sender == this.cboOccurrenceDuration ||
					 sender == this.cboEndTime )
				{
					if ( sender == this.cboOccurrenceStartTime )
					{
						//	When the start time changes, we update the end time		
						this.UpdateEndTime();

						this.appointmentRecurrence.OccurrenceStartTime = this.OccurrenceStartTime;
					}
					else
					if ( sender == this.cboOccurrenceDuration )
					{
						//	When the duration changes, we update the end time
						
						//	Throw an exception if the new duration is invalid
						TimeSpan shortestDuration = Utilities.GetShortestDurationFromDaysOfWeek( this.PatternDaysOfWeek );
						TimeSpan newDuration = this.OccurrenceDuration;
						if ( shortestDuration.TotalMinutes < newDuration.TotalMinutes )
						{
							displayGenericError = false;
							throw new Exception( RecurrenceDialog2007.RecurrenceDialog_Error_DurationCannotExceedFrequency );
						}

						//	Set the end time combo's new text
						DateTime startTime = this.OccurrenceStartTime;

						//	Repopulate the end time combo
						this.PopulateEndTimeCombo( startTime );

						this.cboEndTime.Text = startTime.Add( newDuration ).ToString(Utilities.TimeFormatString);															  
					}
					else
					if ( sender == this.cboEndTime )
					{
						//	When the end time changes, we update the duration
						this.UpdateOccurrenceDuration();
					}
				}
				#endregion StartTime / EndTime / Duration ComboBoxes

				switch( recurrencePropId )
				{
					case PropertyIds.PatternDayOfMonth:
					{
						if ( this.PatternDayOfMonth > 28 && this.PatternDayOfMonth < 32 )
						{
							this.patternDayOfMonthWarningDisplayed = true;

							MessageBox.Show( RecurrenceDialog2007.RecurrenceDialog_Warning_PatternDayOfMonthExceeds28, RecurrenceDialog2007.RecurrenceDialog_MessageBox_Caption, MessageBoxButtons.OK, MessageBoxIcon.Exclamation );
						}

						this.appointmentRecurrence.PatternDayOfMonth = this.PatternDayOfMonth;
					}
					break;

					case PropertyIds.PatternDaysOfWeek:
					{
						TimeSpan shortestDuration = Utilities.GetShortestDurationFromDaysOfWeek( this.PatternDaysOfWeek );
						if ( shortestDuration.TotalMinutes < this.OccurrenceDuration.TotalMinutes )
						{
							displayGenericError = false;
							throw new Exception( RecurrenceDialog2007.RecurrenceDialog_Error_DurationCannotExceedFrequency );
						}

						this.appointmentRecurrence.PatternDaysOfWeek = this.PatternDaysOfWeek;
					}
					break;

					case PropertyIds.PatternFrequency:
					{
						this.appointmentRecurrence.PatternFrequency = this.PatternFrequency;
					}
					break;

					case PropertyIds.PatternInterval:
					{
						this.appointmentRecurrence.PatternInterval = this.PatternInterval;
					}
					break;

					case PropertyIds.PatternMonthOfYear:
					{
						this.appointmentRecurrence.PatternMonthOfYear = this.PatternMonthOfYear;
					}
					break;

					case PropertyIds.PatternOccurrenceOfDayInMonth:
					{
						this.appointmentRecurrence.PatternOccurrenceOfDayInMonth = this.PatternOccurrenceOfDayInMonth;
					}
					break;

					case PropertyIds.PatternType:
					{
						this.appointmentRecurrence.PatternType = this.PatternType;
					}
					break;

					case PropertyIds.RangeEndDate:
					{
						this.appointmentRecurrence.RangeEndDate = this.RangeEndDate;
					}
					break;

					case PropertyIds.RangeLimit:
					{
						this.appointmentRecurrence.RangeLimit = this.RangeLimit;
					}
					break;

					case PropertyIds.RangeMaxOccurrences:
					{
						this.appointmentRecurrence.RangeMaxOccurrences = this.RangeMaxOccurrences;
					}
					break;

					case PropertyIds.RangeStartDate:
					{
						this.appointmentRecurrence.RangeStartDate = this.RangeStartDate;
					}
					break;
				}

				if ( recurrencePropId != PropertyIds.RangeEndDate )
				{
					this.UpdateRangeEndDate();
				}
			}
			catch( Exception ex )
			{
				this.contentsAreValid = false;

				//	Some errors are more specific than "The recurrence pattern is not valid";
				//	in these cases, display the exception message.
				string errorMessage =	displayGenericError ? 
										RecurrenceDialog2007.RecurrenceDialog_Error_RecurrencePatterIsInvalid :
										ex.Message;

				//	Inform the end user of the error condition.
				MessageBox.Show( errorMessage,
								 RecurrenceDialog2007.RecurrenceDialog_MessageBox_Caption,
								 MessageBoxButtons.OK,
								 MessageBoxIcon.Exclamation );

				//	Select the dialog control
				sender.Select();

				//	For dialog controls that support selectable text, select all the text.
				UltraTextEditor textBox = sender as UltraTextEditor;
				UltraComboEditor comboBox = sender as UltraComboEditor;

				if ( textBox != null )
					textBox.SelectAll();
				else
				if ( comboBox != null && comboBox.DropDownStyle == DropDownStyle.DropDown )
					comboBox.SelectAll();
			}
		}

			#endregion OnLeaveDialogControl

			#region Command Button Click event handlers

			#region Cancel
		private void cmdCancel_Click(object sender, System.EventArgs e)
		{
			this.forceContentsChanged = false;
			this.Close();
		}
			#endregion Cancel

			#region Remove Recurrence
		private void cmdRemoveRecurrence_Click(object sender, System.EventArgs e)
		{
			this.result = RecurrenceDialogResult.RemoveRecurrence;
			this.Close();
		}
			#endregion Remove Recurrence

			#region Ok
		private void cmdOk_Click(object sender, System.EventArgs e)
		{
			if ( this.ActiveControl != null )
			{
				this.OnLeaveDialogControl( this.ActiveControl, new EventArgs() );
				if ( this.contentsAreValid == false )
					return;
			}

			this.result = RecurrenceDialogResult.Ok;

			if ( this.PatternDayOfMonth > 28 &&
				 this.patternDayOfMonthWarningDisplayed == false )
				MessageBox.Show( RecurrenceDialog2007.RecurrenceDialog_Warning_PatternDayOfMonthExceeds28, RecurrenceDialog2007.RecurrenceDialog_MessageBox_Caption, MessageBoxButtons.OK, MessageBoxIcon.Exclamation );

			bool close = true;

			//	If the  recurrence we are modifying has variances, we must
			//	warn the end user about the potential loss of data
			//	and prompt as to whether to continue.
			if ( this.hasVariances &&
				 ! Utilities.AreRecurrencesEqual(this.initialAppointmentRecurrence, this.appointmentRecurrence) )
			{
				DialogResult result =	MessageBox.Show( RecurrenceDialog2007.RecurrenceDialog_Prompt_OkToRemoveVariances,
										RecurrenceDialog2007.RecurrenceDialog_MessageBox_Caption,
										MessageBoxButtons.OKCancel,
										MessageBoxIcon.Warning );

				if ( result == DialogResult.Cancel )
				{
					this.result = RecurrenceDialogResult.Cancel;
					close = false;
				}
			}

			if ( close )
				this.Close();
		}
			#endregion Ok

			#endregion Command Button Click event handlers

		#endregion Event handlers

//		#region String resource support
//		private string GetResourceString( string resourceName )
//		{
//			ResourceManager rm = ResourceManager.CreateFileBasedResourceManager("strings.resx", "C:\\", null);
//			return rm.GetObject(resourceName).ToString();
//		}
//		#endregion String resource support
//
	}
	#endregion RecurrenceDialog class

	#region Utilities
	/// <summary>
	/// Class that exposes static utility methods
	/// </summary>
	internal class Utilities
	{
		#region TimeFormatString
#if DEBUG
		/// <summary>
		/// Returns the time format string from the system's LongTimePattern, stripped of the seconds component.
		/// </summary>
#endif
		internal static string TimeFormatString
		{
			get
			{
				string formatString = System.Globalization.CultureInfo.CurrentCulture.DateTimeFormat.LongTimePattern;
				
				formatString = formatString.Replace( ":s", string.Empty );
				formatString = formatString.Replace( "s", string.Empty );
				
				return formatString;
			}
		}
		#endregion TimeFormatString

		#region GetTotalNumberOfDaysInMonth
#if DEBUG
		/// <summary>
		/// Returns the total number of days in the month corresponding to the specified date.
		/// </summary>
		/// <param name="date">The DateTime containing the month to test.</param>
#endif
		static internal int GetTotalNumberOfDaysInMonth( DateTime date )
		{
			System.Globalization.GregorianCalendar cal = new System.Globalization.GregorianCalendar();
			return cal.GetDaysInMonth( date.Year, date.Month );
		}
		#endregion GetTotalNumberOfDaysInMonth

		#region DaysOfWeek
#if DEBUG
		/// <summary>
		/// Returns a string array containing the names of the days of the week for the current culture.
		/// </summary>
#endif
		static internal string[] DaysOfWeek
		{
			get{ return System.Globalization.CultureInfo.CurrentCulture.DateTimeFormat.DayNames;}
		}
		#endregion DaysOfWeek

		#region MonthsOfYear
#if DEBUG
		/// <summary>
		/// Returs a string array containing the names of the months of the year for the current culture.
		/// </summary>
#endif
		static internal string[] MonthsOfYear
		{
			get
			{ 
				string[] retVal = new string[12];
				string[] monthNames =  System.Globalization.CultureInfo.CurrentCulture.DateTimeFormat.MonthNames;
				for ( int i = 0; i < 12; i ++ ){ retVal[i] = monthNames[i]; }
				return retVal;
			}
		}
		#endregion MonthsOfYear

		#region FormatTimeSpan
#if DEBUG
		/// <summary>
		/// Returns the format string to be used for dates.
		/// </summary>
#endif
		static internal string FormatTimeSpan( TimeSpan timeSpan, bool encloseInParentheses )
		{
			double totalMinutes = timeSpan.TotalMinutes;
			double totalHours = Math.Abs(timeSpan.TotalHours);
			double totalDays = Math.Abs(timeSpan.TotalDays);
			double totalWeeks = Math.Abs(totalDays / 7.0);
			double totalMonths = Math.Abs(totalDays / 30.0);

			string retVal = string.Empty;

			if ( totalMinutes == 0.0f )
				retVal = "0 minutes";

			string formatSpecifier = "f0";
			if ( Utilities.IsFormattable(totalMonths, out formatSpecifier) )
			{
				retVal = totalMonths.ToString(formatSpecifier) + " month";
				if ( totalMonths > 1.0f ){ retVal += "s"; }
			}
			else
			if ( Utilities.IsFormattable(totalWeeks, out formatSpecifier) )
			{
				retVal = totalWeeks.ToString(formatSpecifier) + " week";
				if ( totalWeeks > 1.0f ){ retVal += "s"; }
			}
			else
			if ( Utilities.IsFormattable(totalDays, out formatSpecifier) )
			{
				retVal = totalDays.ToString(formatSpecifier) + " day";
				if ( totalDays > 1.0f ){ retVal += "s"; }
			}
			else
			if ( Utilities.IsFormattable(totalHours, out formatSpecifier) )
			{
				retVal = totalHours.ToString(formatSpecifier) + " hour";
				if ( totalHours > 1.0f ){ retVal += "s"; }
			}
			else
			{
				retVal = totalMinutes.ToString("f0") + " minute";
				if ( totalMinutes > 1.0f ){ retVal += "s"; }
			}

			if ( encloseInParentheses )
				retVal = "(" + retVal + ")";

			return retVal;
		}
		#endregion FormatTimeSpan

		#region IsFormattable
#if DEBUG
		/// <summary>
		/// Returns whether the specified value is formattable as either
		/// a whole number, x.25, x.5, or x.75. Also returns the appropriate
		/// format specifier if the method returns true
		/// </summary>
		/// <param name="value">The double-precision value to test</param>
		/// <param name="formatSpecifier">[out] If the method returns true, contains the appropriate format specifier.</param>
		/// <returns></returns>
#endif
		static private bool IsFormattable( double value, out string formatSpecifier )
		{
			formatSpecifier = "f0";
			bool retVal = false;

			if ( value < 1.0f )
				return false;

			if ( (value % 1.0f) == 0.0f )
			{
				retVal = true;
				formatSpecifier = "f0";
			}
			else
			if ( (value % 0.5f) == 0.0f )
			{
				retVal = true;
				formatSpecifier = "f1";
			}
			else
			if ( (value % 0.25f) == 0.0f ||
				 (value % 0.75f) == 0.0f )
			{
				retVal = true;
				formatSpecifier = "f2";
			}

			return retVal;


		}
		#endregion IsFormattable

		#region ParseTimeSpan
#if DEBUG
		/// <summary>
		/// Returns true if the specified string could be successfully parsed
		/// into a TimeSpan; otherwise, returns false. If successful, the 'timeSpan'
		/// parameter contains a valid value.
		/// </summary>
#endif
		static internal bool ParseTimeSpan( string value, out TimeSpan timeSpan )
		{
			timeSpan = TimeSpan.Zero;

			if ( value == null )
				return false;

			if ( value.IndexOf( "minute" ) > -1 ||
				 value.IndexOf( "hour" ) > -1	||
				 value.IndexOf( "day" ) > -1	||
				 value.IndexOf( "week" ) > -1	||
				 value.IndexOf( "month" ) > -1	||
				 value.IndexOf( "year" ) > -1 )
			{
				string temp = value;
				bool unitIsMinute = false;
				bool unitIsHour = false;
				bool unitIsDay = false;
				bool unitIsWeek = false;
				bool unitIsMonth = false;
				bool unitIsYear = false;

				if ( value.IndexOf( "minute" ) > -1 )
				{
					temp = temp.Replace( "minute", string.Empty );
					unitIsMinute = true;
				}
				else
				if ( value.IndexOf( "hour" ) > -1 )
				{
					temp = temp.Replace( "hour", string.Empty );
					unitIsHour = true;
				}
				else
				if ( value.IndexOf( "day" ) > -1 )
				{
					temp = temp.Replace( "day", string.Empty );
					unitIsDay = true;
				}
				else
				if ( value.IndexOf( "week" ) > -1 )
				{
					temp = temp.Replace( "week", string.Empty );
					unitIsWeek = true;
				}
				else
				if ( value.IndexOf( "month" ) > -1 )
				{
					temp = temp.Replace( "month", string.Empty );
					unitIsMonth = true;
				}
				else
				if ( value.IndexOf( "year" ) > -1 )
				{
					temp = temp.Replace( "year", string.Empty );
					unitIsYear = true;
				}

				//	Lose the pluralizer, and any spaces
				temp = temp.Replace( "s", string.Empty );
				temp = temp.Replace( " ", string.Empty );

				try
				{
					double theNumber = Convert.ToDouble( temp );
					if ( unitIsMinute )
						timeSpan = TimeSpan.FromMinutes( theNumber );
					else
					if ( unitIsHour )
						timeSpan = TimeSpan.FromHours( theNumber );
					else
					if ( unitIsDay )
						timeSpan = TimeSpan.FromDays( theNumber );
					else
					if ( unitIsWeek )
						timeSpan = TimeSpan.FromDays( theNumber * 7.0 );
					else
					if ( unitIsMonth )
						timeSpan = TimeSpan.FromDays( theNumber * 30.0 );
					else
					if ( unitIsYear )
						timeSpan = TimeSpan.FromDays( theNumber * 365.0 );

					return true;
				}
				catch
				{
					return false;
				}
			}

			return false;

		}
		#endregion ParseTimeSpan

		#region GetRecurrencePatternDaysOfWeekFromDayOfWeek
#if DEBUG
		/// <summary>
		/// Returns the RecurrencePatternDaysOfWeek constant that corresponds to
		/// the specified System.DayOfWeek constant.
		/// </summary>
		/// <param name="dayOfWeek">The day of the week to return.</param>
		/// <returns>A RecurrencePatternDaysOfWeek value that matches the specified System.DayOfWeek</returns>
#endif
		static internal RecurrencePatternDaysOfWeek GetRecurrencePatternDaysOfWeekFromDayOfWeek( System.DayOfWeek dayOfWeek )
		{
			int dayInWeek = (int)dayOfWeek;
			int retVal = (int)(System.Math.Pow( 2F, (double)dayInWeek ));
			return (RecurrencePatternDaysOfWeek)retVal;
		}
		#endregion GetRecurrencePatternDaysOfWeekFromDayOfWeek

		#region GetOccurrenceOfDayInMonth
#if DEBUG
		/// <summary>
		/// Returns a RecurrencePatternOccurrenceOfDayInMonth value from the specified DateTime.
		/// </summary>
		/// <param name="date">The date for which to obtain the RecurrencePatternOccurrenceOfDayInMonth value.</param>
		/// <returns>A RecurrencePatternDaysOfWeek value that matches the specified System.DayOfWeek</returns>
#endif
		static internal RecurrencePatternOccurrenceOfDayInMonth GetOccurrenceOfDayInMonth( DateTime date )
		{
			//	Get the first day of the month that was specified
			DateTime startDate = new DateTime( date.Year, date.Month, 1, 0, 0, 0 );
			
			//	Move to the first occurrence of the specified day of week
			for ( int i = 1; i <= 7; i ++ )
			{
				if ( startDate.DayOfWeek == date.DayOfWeek )
					break;

				startDate = startDate.AddDays(1f);
			}

			if ( startDate.Date == date.Date )
				return RecurrencePatternOccurrenceOfDayInMonth.First;
			
			DateTime temp = startDate.Date.AddDays( 7f );
			if ( temp.Month != date.Month )
				return RecurrencePatternOccurrenceOfDayInMonth.Last;
			else
			if ( temp == date.Date )
				return RecurrencePatternOccurrenceOfDayInMonth.Second;
			
			temp = startDate.Date.AddDays( 14f );
			if ( temp.Month != date.Month )
				return RecurrencePatternOccurrenceOfDayInMonth.Last;
			else
			if ( temp == date.Date )
				return RecurrencePatternOccurrenceOfDayInMonth.Third;
			
			temp = startDate.Date.AddDays( 21f );
			if ( temp.Month != date.Month )
				return RecurrencePatternOccurrenceOfDayInMonth.Last;
			else
			if ( temp == date.Date )
				return RecurrencePatternOccurrenceOfDayInMonth.Fourth;

			return RecurrencePatternOccurrenceOfDayInMonth.Last;
			
		}
		#endregion GetOccurrenceOfDayInMonth

		#region GetShortestDurationFromDaysOfWeek
#if DEBUG
		/// <summary>
		/// Returns the shortest duration between adjacent occurrences in the specified RecurrencePatternDaysOfWeek.
		/// 
		/// </summary>
		/// <param name="daysOfWeek"></param>
		/// <returns>A TimeSpan representing the shortest duration between any 2 occurrences.</returns>
#endif
		static internal TimeSpan GetShortestDurationFromDaysOfWeek( RecurrencePatternDaysOfWeek daysOfWeek )
		{
			//	Initialize an array of 14 booleans to false
			bool[] days = new bool[14];
			for ( int i = 0; i < 14; i ++ ){ days[i] = false; }

			//	Based on which bits are set in the argument, set
			//	the corresponding member of the array to true
			for ( int i = 0; i < 7; i ++ )
			{
				int bitMask = (int)(System.Math.Pow( 2F, (double)i ));
				days[i] = ((int)daysOfWeek & bitMask) == bitMask ;
			}

			//	Duplicate the first 7 members of the array...this is
			//	to get a sample that is 2 weeks in duration, so we account
			//	for wrapping around to the following week
			for ( int i = 7; i < 14; i ++ )
			{
				days[i] = days[i-7];
			}

			//	Iterate the array and get the 2 members that are "closest together"
			int lastDaySet = -1;
			int shortestDuration = 7;
			for ( int i = 0; i < 14; i ++ )
			{
				//	If not set, skip this iteration
				if ( ! days[i] )
					continue;

				//	Set the initial value of the 'lastDaySet' stack variable
				if ( days[i] && lastDaySet == -1 )
				{
					lastDaySet = i;
					continue;
				}

				//	Each time we hit a set bit in the mask, see how far away
				//	it is from the previous one, and update the 'shortestDuration'
				//	stack variable if this pair is closer together than the last.
				if ( days[i] && lastDaySet >= 0 )
				{
					int duration = i - lastDaySet;
					shortestDuration = System.Math.Min( shortestDuration, duration );
					lastDaySet = i;
				}
			}

			//	Return a TimeSpan that represents the number of days
			//	between the 2 closest occurrences.
			return new TimeSpan( shortestDuration, 0, 0, 0 );
		}
		#endregion GetShortestDurationFromDaysOfWeek
		
		#region AreRecurrencesEqual
#if DEBUG
		/// <summary>
		/// Returns whether the property values of 'recurrence1'
		/// are the same as those of 'recurrence2'. Note that the
		/// RootAppointment and contents of the Variances collections
		/// are not considered.
		/// </summary>
		/// <returns>A boolean indicating whether the 2 instances are logically the same.</returns>
#endif
		static internal bool AreRecurrencesEqual( AppointmentRecurrence recurrence1, AppointmentRecurrence recurrence2 )
		{
			if ( recurrence1 == null && recurrence2 == null )
				return true;

			if ( (recurrence1 == null && recurrence2 != null) ||
				 (recurrence2 == null && recurrence1 != null) )
				return false;

			return	recurrence1.OccurrenceStartTime == recurrence2.OccurrenceStartTime &&
					recurrence1.OccurrenceDuration == recurrence2.OccurrenceDuration &&
					recurrence1.PatternDayOfMonth == recurrence2.PatternDayOfMonth &&
					recurrence1.PatternDaysOfWeek == recurrence2.PatternDaysOfWeek &&
					recurrence1.PatternFrequency == recurrence2.PatternFrequency &&
					recurrence1.PatternInterval == recurrence2.PatternInterval &&
					recurrence1.PatternMonthOfYear == recurrence2.PatternMonthOfYear &&
					recurrence1.PatternOccurrenceOfDayInMonth == recurrence2.PatternOccurrenceOfDayInMonth &&
					recurrence1.PatternType == recurrence2.PatternType &&
					recurrence1.RangeEndDate == recurrence2.RangeEndDate &&
					recurrence1.RangeLimit == recurrence2.RangeLimit &&
					recurrence1.RangeMaxOccurrences == recurrence2.RangeMaxOccurrences &&
					recurrence1.RangeStartDate == recurrence2.RangeStartDate;
		}
		#endregion AreRecurrencesEqual


	}
	#endregion Utilities

}
